/*
 * This file is the top level module.
 *     
 * Copyright (C) 2017
 * Authors: Wen Wang <wen.wang.ww349@yale.edu>
 *          Ruben Niederhagen <ruben@polycephaly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
*/

module main
#(
  parameter m = 13,
  parameter t = 119,
  parameter N_R_elim = 1,
  parameter BLOCK_R_elim = 1,
  parameter N_H_elim = 40,
  parameter BLOCK_H_elim = 10,
  parameter K = 6960,
  parameter M_H_elim = 1
)
(
  input wire start, //start&rst
  input wire clk,
  input wire [63:0] seed_0,
  input wire [63:0] seed_1,
  
  // host interface for testing
  input wire host_P_rd_en,
  input wire [m-1:0] host_P_rd_addr,
  output wire [m-1:0] P_dout,
  
  input wire host_R_elim_rd_en,
  input wire [$clog2(t*(t+1)/N_R_elim)-1 : 0] host_R_elim_rd_addr,
  output wire [N_R_elim*m-1:0] R_elim_dout,
  
  input wire host_H_elim_rd_en,
  input wire [$clog2(L*K_elim/N_H_elim) - 1 : 0] host_H_elim_rd_addr,
  output wire [N_H_elim*M_H_elim-1:0] H_elim_dout,
  
  output wire r_gen_done,
  output wire R_elim_init_done,
  output wire R_elim_done,
  output wire H_elim_init_done,
  output wire FYS_done,
  output wire eva_done,
  output wire H_elim_done,
  output wire done
);

   localparam K_elim = N_H_elim*((K+N_H_elim-1)/N_H_elim);
   localparam L = m*t;

   key_gen key_gen_inst (
    .clk(clk),
    .start(start),
    .seed_0(seed_0),
    .seed_1(seed_1),
    .host_P_rd_en(host_P_rd_en),
    .host_P_rd_addr(host_P_rd_addr),
    .P_dout(P_dout),
    .host_H_elim_rd_en(host_H_elim_rd_en),
    .host_H_elim_rd_addr(host_H_elim_rd_addr),
    .H_elim_dout(H_elim_dout),
    .host_R_elim_rd_en(host_R_elim_rd_en),
    .host_R_elim_rd_addr(host_R_elim_rd_addr),
    .R_elim_dout(R_elim_dout),
    .r_gen_done(r_gen_done),
    .R_elim_done(R_elim_done),
    .H_elim_init_done(H_elim_init_done),
    .FYS_done(FYS_done),
    .eva_done(eva_done),
    .H_elim_done(H_elim_done),
    .done(done),
    .R_elim_init_done(R_elim_init_done)
    );

  defparam key_gen_inst.m = 13;
  defparam key_gen_inst.t = 119;
  defparam key_gen_inst.N_R_elim = 1;
  defparam key_gen_inst.N_H_elim = 40;
  defparam key_gen_inst.K = 6960;
  defparam key_gen_inst.M_H_elim = 1;
  defparam key_gen_inst.BLOCK_R_elim = 1;
  defparam key_gen_inst.BLOCK_H_elim = 10;

endmodule

