/*
 * This file is a sub module for the Gaussian Systemizer module.
 *
 * Copyright (C) 2017
 * Authors: Wen Wang <wen.wang.ww349@yale.edu>
 *          Ruben Niederhagen <ruben@polycephaly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
*/

module comb_SA_R_elim (
  input wire functionA,
  input wire first_pass,
  input wire [0:0] pass,
  input wire clk,
  input wire rst,
  input start,
  input finish,
  input wire [14:0] op_in,
  input wire [12:0] data,
  output wire [12:0] data_out,
  output wire [14:0] op_out,
  output wire r_A_and
);

parameter BLOCK = -1;

generate
if (BLOCK != 1) begin
    ERROR_module_parameter_BLOCK_not_set_correctly ASSERT_ERROR();
end
endgenerate

  reg [0 : 0] functionA_dup = 0;

  reg [0 : 1] start_tmp = 0;
  reg [0 : 1] start_row = 0;

  reg [0 : 1] finish_tmp = 0;
  reg [0 : 1] finish_tmp_d = 0;
  reg [0 : 1] finish_in = 0;
  wire [0 : 0] finish_out;

  reg [0 : 0] first_pass_tmp = 0;

  reg [0 : 0] first_pass_row = 0;


  always @(posedge clk) begin
    functionA_dup[0] <= functionA;



  end

  wire first_pass_row_0;
  assign first_pass_row_0 = first_pass;

  wire [12:0] data_delay;
  delay #(.WIDTH(13), .DELAY(0)) del_inst_in_0 (
    .clk(clk),
    .din(data[12:0]),
    .dout(data_delay[12:0])
  );

  /////////////////////////////////////
  // row 0
  // row 0, col 0

  wire start_in_0_0;
  wire [1:0] op_in_0_0;


  wire [12:0] fac_in_0_0;


  wire start_out_0_0;
  wire [1:0] op_out_0_0;


  wire [12:0] fac_out_0_0;

  wire r_0_0;

  wire [12:0] data_in_0_0;
  wire [12:0] data_out_0_0;

  assign data_in_0_0 = data_delay[12:0];

  assign start_in_0_0 = start;
  assign op_in_0_0 = op_in[1:0];
  assign fac_in_0_0 = op_in[14:2];

  processor_AB_R_elim #(.WIDTH(13)) P_0_0 (
    .functionA  (functionA_dup[0]),
    .first_pass (first_pass_row_0),
    .pass       (pass[0]),
    .clk        (clk),
    .rst        (rst),
    .data_in    (data_in_0_0),
    .start_in   (start_in_0_0),
    .finish_in  (finish),
    .finish_out (finish_out[0]),
    .op_in      (op_in_0_0),
    .op_out     (op_out_0_0),
    .fac_in      (fac_in_0_0),
    .fac_out     (fac_out_0_0),
    .start_out  (start_out_0_0),
    .data_out   (data_out_0_0),
    .r          (r_0_0)
  );


  /////////////////////////
  // outputs 


  reg [12:0] data_out_reg = 0;

  always @(posedge clk) begin
    data_out_reg <= {data_out_0_0};
  end

  delay #(.WIDTH(13), .DELAY(0)) del_inst_out_0 (
    .clk(clk),
    .din(data_out_reg[12:0]),
    .dout(data_out[12:0])
  ); 


  assign op_out = {fac_out_0_0, op_out_0_0};

  assign r_A_and = r_0_0;

endmodule

