/*
 * This file is the testbench file for key_gen.v.
 *
 * Copyright (C) 2017
 * Authors: Wen Wang <wen.wang.ww349@yale.edu>
 *          Ruben Niederhagen <ruben@polycephaly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * M_elimERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, M_elimA 02110-1301  USA
 *
*/

`timescale 1ns / 1ps
module key_gen_tb;

  //inputs
  reg clk = 1'b0;
  reg start = 1'b0;
  reg [63:0] seed_0 = 0;
  reg [63:0] seed_1 = 0;
  
  reg host_P_rd_en = 0;
  reg [`m-1:0] host_P_rd_addr = 0;
  reg host_R_elim_rd_en = 0;
  reg [`CLOG2(`t*(`t+1)/`N_R_elim)-1 : 0] host_R_elim_rd_addr = 0;
  reg host_H_elim_rd_en = 0;
  reg [`CLOG2(L*K_elim/`N_H_elim) - 1 : 0] host_H_elim_rd_addr = 0;
  
  //outputs
  
  wire r_gen_done;
  wire R_elim_init_done;
  wire R_elim_done;
  wire H_elim_init_done;
  wire FYS_done;
  wire eva_done;
  wire H_elim_done;
  wire done;
  wire fail;
  
  wire [`N_R_elim*`m-1:0] R_elim_dout;
  wire [`N_H_elim*`M_H_elim-1:0] H_elim_dout;
  wire [`m-1:0] P_dout;
   
   
  localparam K_elim = `N_H_elim*((`K+`N_H_elim-1)/`N_H_elim);
  localparam L = `m*`t;
  
  key_gen #(.m(`m), .t(`t), .N_R_elim(`N_R_elim), .N_H_elim(`N_H_elim), .K(`K), .M_H_elim(`M_H_elim), .BLOCK_R_elim(`BLOCK_R_elim), .BLOCK_H_elim(`BLOCK_H_elim)) DUT (
    .clk(clk),
    .start(start),
    .seed_0(seed_0),
    .seed_1(seed_1),
    .host_P_rd_en(host_P_rd_en),
    .host_P_rd_addr(host_P_rd_addr),
    .P_dout(P_dout),
    .host_H_elim_rd_en(host_H_elim_rd_en),
    .host_H_elim_rd_addr(host_H_elim_rd_addr),
    .H_elim_dout(H_elim_dout),
    .host_R_elim_rd_en(host_R_elim_rd_en),
    .host_R_elim_rd_addr(host_R_elim_rd_addr),
    .R_elim_dout(R_elim_dout),
    .r_gen_done(r_gen_done),
    .R_elim_done(R_elim_done),
    .H_elim_init_done(H_elim_init_done),
    .FYS_done(FYS_done),
    .eva_done(eva_done),
    .R_elim_init_done(R_elim_init_done),
    .H_elim_done(H_elim_done),
    .done(done),
    .fail(fail)
    );
 
//  initial
//    begin
//      $dumpfile("key_gen_tb.vcd");
//      $dumpvars(0, key_gen_tb);
//    end
 
  integer key_gen_start_time;
  integer R_elim_init_start_time;
  integer R_elim_start_time;
  integer eva_start_time;
  integer eva_done_time;
  integer H_elim_start_time;
  integer elim_start_time;
  
  integer STDERR = 32'h8000_0002;
  
  integer round = 0;
  
  initial
    begin
      for (round = 0; round < `rounds; round = round + 1)
      begin
        start <= 0;
        seed_0 <= 98;
        seed_1 <= 1791243;
        # 25;
        key_gen_start_time = $time;
        start <= 1;
        # 10;
        start <= 0;
        
        @(posedge done);

        # 20;
      end

      $display("\n Testing Result:\n   %0d rounds just finished!\n   runtime for generating a key pair: %0d cycles\n", round, ($time-key_gen_start_time)/10);
      $fflush();

      q = $fopen("pk.out","w");

      @(posedge clk);

      # 5;

      host_H_elim_rd_en = 1'b1;

      for (i = L*(L/`N_H_elim); i < L*(K_elim/`N_H_elim); i = i+1)
        begin
          host_H_elim_rd_addr = i;
          # 10;
          $fwrite(q, "%b\n", DUT.H_elim_dout);
        end

      # 5;
      $fclose(q);

      #100;
      $finish;
         
   end

  integer f;
  integer i;
  integer q;

  always
    begin
      @(posedge r_gen_done);
      R_elim_init_start_time = $time;
      $display("\n\n Round: %0d\n generating random r is finished ", round);
      $fflush();
      f = $fopen("r.out","w");

      $fwrite(f, "%b\n", DUT.r_gen_coeff_out);
      $fclose(f);
    end

  always
    begin
      @(posedge R_elim_init_done);
      R_elim_start_time = $time;

      $display("\n running time for R initialization: %0d cycles", ($time-R_elim_init_start_time)/10);
      $fflush();

      $writememb("R_init.out", DUT.R_elim_inst.elim_inst.phase_inst.step_inst.mem_data.mem);
   end

  always
    begin
      @(posedge FYS_done);

      $writememb("FYS.out", DUT.H_elim_inst.gen_H_inst.FYS_inst.mem_dual_inst.mem);
    end

  always
    begin
      @(posedge R_elim_done);
      eva_start_time = $time;

      $display("\n running time for R elimination: %0d cycles", ($time-R_elim_start_time)/10);
      $fflush();

      $writememb("R_elim.out", DUT.R_elim_inst.elim_inst.phase_inst.step_inst.mem_data.mem);
    end

  always
    begin
      @(posedge eva_done);
      eva_done_time = $time;

      $display("\n running time for polynomial evaluation: %0d cycles", ($time-eva_start_time)/10);
      $fflush();

      $writememb("eva_A.out", DUT.H_elim_inst.gen_H_inst.add_FFT_inst.reduction_inst.mem_coeff_A.mem);
      $writememb("eva_B.out", DUT.H_elim_inst.gen_H_inst.add_FFT_inst.reduction_inst.mem_coeff_B.mem);
    end

  always
    begin
      @(posedge H_elim_init_done);
      H_elim_start_time = $time;

      $display("\n running time for H initialization: %0d cycles", ($time-eva_done_time)/10);
      $fflush();

      $writememb("H_init.out", DUT.H_elim_inst.elim_inst.phase_inst.step_inst.mem_data.mem);
    end

  always
    begin
      @(posedge H_elim_done);

      $display("\n running time for H elimination: %0d cycles", ($time-H_elim_start_time)/10);
      $fflush();

      $writememb("H_elim.out", DUT.H_elim_inst.elim_inst.phase_inst.step_inst.mem_data.mem);
    end

  always 
    #5 clk = ! clk; 
   
endmodule

