/*
 * Copyright (C) 2017
 * Authors: Wen Wang <wen.wang.ww349@yale.edu>
 *          Ruben Niederhagen <ruben@polycephaly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
*/


// read out corresponding g(a) values

module point_loc (
  input wire          start,
  input wire          clk,
  input wire [5:0]  location,
  input wire [831:0]  data_in,
  output wire [12:0] data_out,
  output wire         done
);

reg [831:0] data_64_reg = 832'b0;
reg [415:0] data_32_reg = 416'b0;
reg [207:0] data_16_reg = 208'b0;
reg [103:0] data_8_reg = 104'b0;
reg [51:0] data_4_reg = 52'b0;
reg [25:0] data_2_reg = 26'b0;
reg [12:0] data_1_reg = 13'b0;


reg [5:0] location_64_reg = 6'b0;
reg [5:0] location_32_reg = 6'b0;
reg [5:0] location_16_reg = 6'b0;
reg [5:0] location_8_reg = 6'b0;
reg [5:0] location_4_reg = 6'b0;
reg [5:0] location_2_reg = 6'b0;

always @(posedge clk)
  begin
    location_64_reg <= start ? location : location_64_reg;
    data_64_reg <= start ? data_in : data_64_reg;

    data_32_reg <= (location_64_reg >> 5) ? data_64_reg[831:416] : data_64_reg[415:0];
    location_32_reg <= (location_64_reg >> 5) ? (location_64_reg - 32) : location_64_reg;

    data_16_reg <= (location_32_reg >> 4) ? data_32_reg[415:208] : data_32_reg[207:0];
    location_16_reg <= (location_32_reg >> 4) ? (location_32_reg - 16) : location_32_reg;

    data_8_reg <= (location_16_reg >> 3) ? data_16_reg[207:104] : data_16_reg[103:0];
    location_8_reg <= (location_16_reg >> 3) ? (location_16_reg - 8) : location_16_reg;

    data_4_reg <= (location_8_reg >> 2) ? data_8_reg[103:52] : data_8_reg[51:0];
    location_4_reg <= (location_8_reg >> 2) ? (location_8_reg - 4) : location_8_reg;

    data_2_reg <= (location_4_reg >> 1) ? data_4_reg[51:26] : data_4_reg[25:0];
    location_2_reg <= (location_4_reg >> 1) ? (location_4_reg - 2) : location_4_reg;

    data_1_reg <= (location_2_reg >> 0) ? data_2_reg[25:13] : data_2_reg[12:0];

  end

reg done_0 = 1'b0;
reg done_1 = 1'b0;
reg done_2 = 1'b0;
reg done_3 = 1'b0;
reg done_4 = 1'b0;
reg done_5 = 1'b0;
reg done_6 = 1'b0;

always @(posedge clk)
  begin
    done_0 <= start;
    done_1 <= done_0;
    done_2 <= done_1;
    done_3 <= done_2;
    done_4 <= done_3;
    done_5 <= done_4;
    done_6 <= done_5;
  end

assign data_out = data_1_reg;
assign done = done_6;

endmodule

