/*
 * Copyright (C) 2017
 * Authors: Wen Wang <wen.wang.ww349@yale.edu>
 *          Ruben Niederhagen <ruben@polycephaly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
*/

//6-split Karatsuba polynomial multiplication

module poly_mul_koa (
  input  wire            start,
  input  wire	 		       clk,
  input  wire [1559:0]    poly_g,
  input  wire [1559:0]	   poly_f,
  output wire [3106:0]    poly_out,
  output wire			       done,
  output wire            poly_done
);

wire [259:0] g0;
wire [259:0] g1;
wire [259:0] g2;
wire [259:0] g3;
wire [259:0] g4;
wire [259:0] g5;

assign g0 = poly_g[259:0];
assign g1 = poly_g[519:260];
assign g2 = poly_g[779:520];
assign g3 = poly_g[1039:780];
assign g4 = poly_g[1299:1040];
assign g5 = poly_g[1559:1300];

wire [259:0] f0;
wire [259:0] f1;
wire [259:0] f2;
wire [259:0] f3;
wire [259:0] f4;
wire [259:0] f5;

assign f0 = poly_f[259:0];
assign f1 = poly_f[519:260];
assign f2 = poly_f[779:520];
assign f3 = poly_f[1039:780];
assign f4 = poly_f[1299:1040];
assign f5 = poly_f[1559:1300];


reg [4:0] counter = 5'b0;

reg [5:0] gf_mask[0:17];

initial
  begin
    gf_mask[0] = 6'b111111;
    gf_mask[1] = 6'b110110;
    gf_mask[2] = 6'b101101;
    gf_mask[3] = 6'b101001;
    gf_mask[4] = 6'b100101;
    gf_mask[5] = 6'b111000;
    gf_mask[6] = 6'b000111;
    gf_mask[7] = 6'b001100;
    gf_mask[8] = 6'b010010;
    gf_mask[9] = 6'b011000;
    gf_mask[10] = 6'b000110;
    gf_mask[11] = 6'b110000;
    gf_mask[12] = 6'b000011;
    gf_mask[13] = 6'b100000;
    gf_mask[14] = 6'b010000;
    gf_mask[15] = 6'b000010;
    gf_mask[16] = 6'b000001;
    gf_mask[17] = 6'b000000;
  end

wire [5:0] gf_dout;
assign gf_dout = gf_mask[counter];

reg [11:0] out_mask[0:16];

initial
  begin
    out_mask[0] = 12'b000000010000;
    out_mask[1] = 12'b100001010000;
    out_mask[2] = 12'b100011101000;
    out_mask[3] = 12'b100011110000;
    out_mask[4] = 12'b100000101000;
    out_mask[5] = 12'b100100001000;
    out_mask[6] = 12'b100011010100;
    out_mask[7] = 12'b100011011000;
    out_mask[8] = 12'b100001100000;
    out_mask[9] = 12'b100110011000;
    out_mask[10] = 12'b100010011100;
    out_mask[11] = 12'b101101001000;
    out_mask[12] = 12'b100011000110;
    out_mask[13] = 12'b111000110000;
    out_mask[14] = 12'b101011111000;
    out_mask[15] = 12'b100010101010;
    out_mask[16] = 12'b100000110011;
  end
  
wire [11:0] out_dout;
assign out_dout = out_mask[counter];

wire [259:0] g_in;
reg running = 0;
 
assign g_in = (!running) ? 0 : (gf_dout[5] ? g0 : 0) ^ (gf_dout[4] ? g1 : 0) ^ (gf_dout[3] ? g2 : 0) ^ (gf_dout[2] ? g3 : 0) ^ (gf_dout[1] ? g4 : 0) ^ (gf_dout[0] ? g5 : 0);


wire [259:0] f_in;
assign f_in = (!running) ? 0 : (gf_dout[5] ? f0 : 0) ^ (gf_dout[4] ? f1 : 0) ^ (gf_dout[3] ? f2 : 0) ^ (gf_dout[2] ? f3 : 0) ^ (gf_dout[1] ? f4 : 0) ^ (gf_dout[0] ? f5 : 0);


wire [506:0] R_out;

always @(posedge clk)
  begin
    running <= start ? 1'b1 : 
               done ? 1'b0 :
               running;  
  end

always @(posedge clk)
  begin
    counter <= start ? 0 :
               poly_done ? counter + 1 :
               done ? 0 :
               counter;
  end
                 
reg [3106:0] poly_out_buffer = 0;

always @(posedge clk)
  begin
    poly_out_buffer <= start ? 0 :
                       poly_done ? (out_dout[11] ? poly_out_buffer : 0) ^ (out_dout[10] ? (R_out << 0) : 0) ^ (out_dout[9] ? (R_out << 260) : 0) ^ (out_dout[8] ? (R_out << 520) : 0) ^ (out_dout[7] ? (R_out << 780) : 0) ^ (out_dout[6] ? (R_out << 1040) : 0) ^ (out_dout[5] ? (R_out << 1300) : 0) ^ (out_dout[4] ? (R_out << 1560) : 0) ^ (out_dout[3] ? (R_out << 1820) : 0) ^ (out_dout[2] ? (R_out << 2080) : 0) ^ (out_dout[1] ? (R_out << 2340) : 0) ^ (out_dout[0] ? (R_out << 2600) : 0) :
											  poly_out_buffer;
  end

assign poly_out = poly_out_buffer;


reg poly_start = 0;

always @(posedge clk)
  begin
    poly_start <= (start | ((counter < 16) && poly_done));
  end


reg done_buffer = 0;

always @(posedge clk)
  begin
    done_buffer <= ((counter == 16) && poly_done) ? 1'b1 : 1'b0;
  end

assign done = done_buffer;


poly_mul_sb poly_mul_sb_inst (
  .start(poly_start),
  .clk(clk),
  .poly_g(g_in),
  .poly_f(f_in),
  .poly_out(R_out),
  .done(poly_done)
);

endmodule

