/*
 * Copyright (C) 2017
 * Authors: Wen Wang <wen.wang.ww349@yale.edu>
 *          Ruben Niederhagen <ruben@polycephaly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
*/

// 20 gf multipliers used

module poly_mul_sb (
  input  wire          start,
  input  wire	 		     clk,
  input  wire [259:0]  poly_g,
  input  wire [259:0]	 poly_f,
  output wire [506:0]  poly_out,
  output wire  		     done
);

wire [259:0] poly_med;

reg [259:0] poly_g_buffer = 0;

always @(posedge clk)
  begin
    poly_g_buffer <= start ? poly_g : poly_g_buffer;
  end

reg [4:0] counter = 5'b0;
reg running = 1'b0;

wire [12:0] f_i;
reg [259:0] f_shift = 0;

always @(posedge clk)
  begin
    f_shift <= start ? poly_f : (f_shift << 13);
  end

assign f_i = f_shift[259:247];

reg done_buffer = 0;

reg [506:0] poly_out_buffer = 0;

always @(posedge clk)
begin
  running <= start ? 1'b1 :
             done_buffer ? 1'b0 :
             running;

  counter <= start ? 5'b0 : 
             (counter == 20) ? counter :
             running ? counter + 1 : counter;

  poly_out_buffer <= start ? 507'b0: 
                     (counter == 20) ? poly_out_buffer : 
                     ((poly_out_buffer << 13) ^ poly_med);
end

assign poly_out = poly_out_buffer;


always @(posedge clk)
  begin
    done_buffer <= (counter == 19) ? 1 : 0;
  end

assign done = done_buffer;

gf_mul gf_mul_0_inst (
  .clk(clk),
  .dinA(poly_g_buffer[12:0]),
  .dinB(f_i),
  .dout(poly_med[12:0])
);
	
gf_mul gf_mul_1_inst (
  .clk(clk),
  .dinA(poly_g_buffer[25:13]),
  .dinB(f_i),
  .dout(poly_med[25:13])
);
	
gf_mul gf_mul_2_inst (
  .clk(clk),
  .dinA(poly_g_buffer[38:26]),
  .dinB(f_i),
  .dout(poly_med[38:26])
);
	
gf_mul gf_mul_3_inst (
  .clk(clk),
  .dinA(poly_g_buffer[51:39]),
  .dinB(f_i),
  .dout(poly_med[51:39])
);
	
gf_mul gf_mul_4_inst (
  .clk(clk),
  .dinA(poly_g_buffer[64:52]),
  .dinB(f_i),
  .dout(poly_med[64:52])
);
	
gf_mul gf_mul_5_inst (
  .clk(clk),
  .dinA(poly_g_buffer[77:65]),
  .dinB(f_i),
  .dout(poly_med[77:65])
);
	
gf_mul gf_mul_6_inst (
  .clk(clk),
  .dinA(poly_g_buffer[90:78]),
  .dinB(f_i),
  .dout(poly_med[90:78])
);
	
gf_mul gf_mul_7_inst (
  .clk(clk),
  .dinA(poly_g_buffer[103:91]),
  .dinB(f_i),
  .dout(poly_med[103:91])
);
	
gf_mul gf_mul_8_inst (
  .clk(clk),
  .dinA(poly_g_buffer[116:104]),
  .dinB(f_i),
  .dout(poly_med[116:104])
);
	
gf_mul gf_mul_9_inst (
  .clk(clk),
  .dinA(poly_g_buffer[129:117]),
  .dinB(f_i),
  .dout(poly_med[129:117])
);
	
gf_mul gf_mul_10_inst (
  .clk(clk),
  .dinA(poly_g_buffer[142:130]),
  .dinB(f_i),
  .dout(poly_med[142:130])
);
	
gf_mul gf_mul_11_inst (
  .clk(clk),
  .dinA(poly_g_buffer[155:143]),
  .dinB(f_i),
  .dout(poly_med[155:143])
);
	
gf_mul gf_mul_12_inst (
  .clk(clk),
  .dinA(poly_g_buffer[168:156]),
  .dinB(f_i),
  .dout(poly_med[168:156])
);
	
gf_mul gf_mul_13_inst (
  .clk(clk),
  .dinA(poly_g_buffer[181:169]),
  .dinB(f_i),
  .dout(poly_med[181:169])
);
	
gf_mul gf_mul_14_inst (
  .clk(clk),
  .dinA(poly_g_buffer[194:182]),
  .dinB(f_i),
  .dout(poly_med[194:182])
);
	
gf_mul gf_mul_15_inst (
  .clk(clk),
  .dinA(poly_g_buffer[207:195]),
  .dinB(f_i),
  .dout(poly_med[207:195])
);
	
gf_mul gf_mul_16_inst (
  .clk(clk),
  .dinA(poly_g_buffer[220:208]),
  .dinB(f_i),
  .dout(poly_med[220:208])
);
	
gf_mul gf_mul_17_inst (
  .clk(clk),
  .dinA(poly_g_buffer[233:221]),
  .dinB(f_i),
  .dout(poly_med[233:221])
);
	
gf_mul gf_mul_18_inst (
  .clk(clk),
  .dinA(poly_g_buffer[246:234]),
  .dinB(f_i),
  .dout(poly_med[246:234])
);
	
gf_mul gf_mul_19_inst (
  .clk(clk),
  .dinA(poly_g_buffer[259:247]),
  .dinB(f_i),
  .dout(poly_med[259:247])
);
	
endmodule

