##
 # This file checks if the parameters are valid.
 #     
 # Copyright (C) 2017
 # Authors: Wen Wang <wen.wang.ww349@yale.edu>
 #          Ruben Niederhagen <ruben@polycephaly.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 #
##




import sys
import math

import argparse

parser = argparse.ArgumentParser(description='Generate memory contents for H.',
                formatter_class=argparse.ArgumentDefaultsHelpFormatter)

parser.add_argument('-m', '--gf', dest='m', type=int, default=13,
          help='field GF(2^m); default = GF(2^13)')
parser.add_argument('-t', dest='t', type=int, required = True, default=119,  
          help='number of errors; default = 119')
parser.add_argument('-n', dest='n', type=int, required = True, default=8192,  
          help='code length') 
parser.add_argument('-N_H', dest='N_H', type=int, required = True, default=20,  
          help='column block size in H elimination') 
parser.add_argument('-N_R', dest='N_R', type=int, required = True, default=20,  
          help='column block size in R elimination')
parser.add_argument('-BLOCK_H', dest='BLOCK_H', type=int, required = True, default=20,  
          help='BLOCK value in H elimination') 
parser.add_argument('-BLOCK_R', dest='BLOCK_R', type=int, required = True, default=20,  
          help='BLOCK value in R elimination')  

args = parser.parse_args()

m = args.m
t = args.t
n = args.n
N_H = args.N_H
N_R = args.N_R
BLOCK_H = args.BLOCK_H
BLOCK_R = args.BLOCK_R

assert (n <= math.pow(2, m)), "\n\n  Code length n should be no larger than 2^m!\n"

assert (n > m*t), "\n\n  Code length n should be larger than m*t!"

assert (N_H >= BLOCK_H), "\n\n  BLOCK in H elimination can not be larger than N!"
assert (N_R >= BLOCK_R), "\n\n  BLOCK in R elimination can not be larger than N!"

assert ((N_H % BLOCK_H) == 0), "\n\n  N should be a multiple of BLOCK for H elimination!"
assert ((N_R % BLOCK_R) == 0), "\n\n  N should be a multiple of BLOCK for R elimination!"

