/*
 * This file is a sub_module for key_gen.v, which generates a random polynomial of degree-(t-1).
 *
 * Copyright (C) 2017
 * Authors: Wen Wang <wen.wang.ww349@yale.edu>
 *          Ruben Niederhagen <ruben@polycephaly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
*/

module r_gen
#(
  parameter m = 13,
  parameter t = 119
)
(
  input wire start,
  input wire clk,
  input wire [63:0] seed,
  output wire [t*m-1:0] coeff_out,
  output wire done
);
  
  wire [63:0] PRG;
  wire [m-1:0] PRG_low;
  assign PRG_low = PRG[m-1:0];
  
  reg [`CLOG2(t):0] counter = 0;
  reg [t*m-1:0] coeff_out_buffer = 0;
  reg done_buffer = 0;
  reg running = 0;
  
  always @(posedge clk)
    begin
      running <= start ? 1 : running;
      counter <= start ? 0 : 
                 (running && (counter <= t)) ? counter + 1 : counter;
      coeff_out_buffer <= start ? 0 :
                          (counter <= t) ? ((coeff_out_buffer << m) ^ PRG_low) : coeff_out_buffer; 
      done_buffer <= start ? 0 : (counter == t);
    end
  
  assign coeff_out = coeff_out_buffer;
  assign done = done_buffer;
  
  XorShift64 Xorshift64_inst(
    .clk(clk),
    .rst(start),
    .seed(seed),
    .rngout(PRG)
  );
  
endmodule

