/*
 * Copyright (C) 2017
 * Authors: Wen Wang <wen.wang.ww349@yale.edu>
 *          Ruben Niederhagen <ruben@polycephaly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
*/


module rad_conv (
  input wire          clk,
  input wire          start,
  input wire [1663:0]  poly_in,
  input wire [2:0]    round,
  output wire [1663:0] poly_out,
  output wire         done 
);

wire [1663:0] poly_twist_in_1;
wire [1663:0] poly_twist_in_2;
wire [1663:0] poly_twist_in_3;
wire [1663:0] poly_twist_in_4;
wire [1663:0] poly_twist_in_5;
wire [1663:0] poly_twist_in_6;

reg [1663:0] poly_twist_in_1_buffer = 1664'b0;
reg [1663:0] poly_twist_in_2_buffer = 1664'b0;
reg [1663:0] poly_twist_in_3_buffer = 1664'b0;
reg [1663:0] poly_twist_in_4_buffer = 1664'b0;
reg [1663:0] poly_twist_in_5_buffer = 1664'b0;
reg [1663:0] poly_twist_in_6_buffer = 1664'b0;

// output for the 6_th sub radix conversion routine
assign poly_twist_in_1[1663:1651] = poly_in[1663:1651];
assign poly_twist_in_1[1650:1638] = poly_in[1650:1638];
assign poly_twist_in_1[1637:1625] = poly_in[1637:1625];
assign poly_twist_in_1[1624:1612] = poly_in[1624:1612];
assign poly_twist_in_1[1611:1599] = poly_in[1611:1599];
assign poly_twist_in_1[1598:1586] = poly_in[1598:1586];
assign poly_twist_in_1[1585:1573] = poly_in[1585:1573];
assign poly_twist_in_1[1572:1560] = poly_in[1572:1560];
assign poly_twist_in_1[1559:1547] = poly_in[1559:1547];
assign poly_twist_in_1[1546:1534] = poly_in[1546:1534];
assign poly_twist_in_1[1533:1521] = poly_in[1533:1521];
assign poly_twist_in_1[1520:1508] = poly_in[1520:1508];
assign poly_twist_in_1[1507:1495] = poly_in[1507:1495];
assign poly_twist_in_1[1494:1482] = poly_in[1494:1482];
assign poly_twist_in_1[1481:1469] = poly_in[1481:1469];
assign poly_twist_in_1[1468:1456] = poly_in[1468:1456];
assign poly_twist_in_1[1455:1443] = poly_in[1455:1443];
assign poly_twist_in_1[1442:1430] = poly_in[1442:1430];
assign poly_twist_in_1[1429:1417] = poly_in[1429:1417];
assign poly_twist_in_1[1416:1404] = poly_in[1416:1404];
assign poly_twist_in_1[1403:1391] = poly_in[1403:1391];
assign poly_twist_in_1[1390:1378] = poly_in[1390:1378];
assign poly_twist_in_1[1377:1365] = poly_in[1377:1365];
assign poly_twist_in_1[1364:1352] = poly_in[1364:1352];
assign poly_twist_in_1[1351:1339] = poly_in[1351:1339];
assign poly_twist_in_1[1338:1326] = poly_in[1338:1326];
assign poly_twist_in_1[1325:1313] = poly_in[1325:1313];
assign poly_twist_in_1[1312:1300] = poly_in[1312:1300];
assign poly_twist_in_1[1299:1287] = poly_in[1299:1287];
assign poly_twist_in_1[1286:1274] = poly_in[1286:1274];
assign poly_twist_in_1[1273:1261] = poly_in[1273:1261];
assign poly_twist_in_1[1260:1248] = poly_in[1260:1248];
assign poly_twist_in_1[1247:1235] = poly_in[1247:1235] ^ poly_in[1663:1651];
assign poly_twist_in_1[1234:1222] = poly_in[1234:1222] ^ poly_in[1650:1638];
assign poly_twist_in_1[1221:1209] = poly_in[1221:1209] ^ poly_in[1637:1625];
assign poly_twist_in_1[1208:1196] = poly_in[1208:1196] ^ poly_in[1624:1612];
assign poly_twist_in_1[1195:1183] = poly_in[1195:1183] ^ poly_in[1611:1599];
assign poly_twist_in_1[1182:1170] = poly_in[1182:1170] ^ poly_in[1598:1586];
assign poly_twist_in_1[1169:1157] = poly_in[1169:1157] ^ poly_in[1585:1573];
assign poly_twist_in_1[1156:1144] = poly_in[1156:1144] ^ poly_in[1572:1560];
assign poly_twist_in_1[1143:1131] = poly_in[1143:1131] ^ poly_in[1559:1547];
assign poly_twist_in_1[1130:1118] = poly_in[1130:1118] ^ poly_in[1546:1534];
assign poly_twist_in_1[1117:1105] = poly_in[1117:1105] ^ poly_in[1533:1521];
assign poly_twist_in_1[1104:1092] = poly_in[1104:1092] ^ poly_in[1520:1508];
assign poly_twist_in_1[1091:1079] = poly_in[1091:1079] ^ poly_in[1507:1495];
assign poly_twist_in_1[1078:1066] = poly_in[1078:1066] ^ poly_in[1494:1482];
assign poly_twist_in_1[1065:1053] = poly_in[1065:1053] ^ poly_in[1481:1469];
assign poly_twist_in_1[1052:1040] = poly_in[1052:1040] ^ poly_in[1468:1456];
assign poly_twist_in_1[1039:1027] = poly_in[1039:1027] ^ poly_in[1455:1443];
assign poly_twist_in_1[1026:1014] = poly_in[1026:1014] ^ poly_in[1442:1430];
assign poly_twist_in_1[1013:1001] = poly_in[1013:1001] ^ poly_in[1429:1417];
assign poly_twist_in_1[1000:988] = poly_in[1000:988] ^ poly_in[1416:1404];
assign poly_twist_in_1[987:975] = poly_in[987:975] ^ poly_in[1403:1391];
assign poly_twist_in_1[974:962] = poly_in[974:962] ^ poly_in[1390:1378];
assign poly_twist_in_1[961:949] = poly_in[961:949] ^ poly_in[1377:1365];
assign poly_twist_in_1[948:936] = poly_in[948:936] ^ poly_in[1364:1352];
assign poly_twist_in_1[935:923] = poly_in[935:923] ^ poly_in[1351:1339];
assign poly_twist_in_1[922:910] = poly_in[922:910] ^ poly_in[1338:1326];
assign poly_twist_in_1[909:897] = poly_in[909:897] ^ poly_in[1325:1313];
assign poly_twist_in_1[896:884] = poly_in[896:884] ^ poly_in[1312:1300];
assign poly_twist_in_1[883:871] = poly_in[883:871] ^ poly_in[1299:1287];
assign poly_twist_in_1[870:858] = poly_in[870:858] ^ poly_in[1286:1274];
assign poly_twist_in_1[857:845] = poly_in[857:845] ^ poly_in[1273:1261];
assign poly_twist_in_1[844:832] = poly_in[844:832] ^ poly_in[1260:1248];
assign poly_twist_in_1[831:819] = poly_in[831:819] ^ poly_in[1247:1235] ^ poly_in[1663:1651];
assign poly_twist_in_1[818:806] = poly_in[818:806] ^ poly_in[1234:1222] ^ poly_in[1650:1638];
assign poly_twist_in_1[805:793] = poly_in[805:793] ^ poly_in[1221:1209] ^ poly_in[1637:1625];
assign poly_twist_in_1[792:780] = poly_in[792:780] ^ poly_in[1208:1196] ^ poly_in[1624:1612];
assign poly_twist_in_1[779:767] = poly_in[779:767] ^ poly_in[1195:1183] ^ poly_in[1611:1599];
assign poly_twist_in_1[766:754] = poly_in[766:754] ^ poly_in[1182:1170] ^ poly_in[1598:1586];
assign poly_twist_in_1[753:741] = poly_in[753:741] ^ poly_in[1169:1157] ^ poly_in[1585:1573];
assign poly_twist_in_1[740:728] = poly_in[740:728] ^ poly_in[1156:1144] ^ poly_in[1572:1560];
assign poly_twist_in_1[727:715] = poly_in[727:715] ^ poly_in[1143:1131] ^ poly_in[1559:1547];
assign poly_twist_in_1[714:702] = poly_in[714:702] ^ poly_in[1130:1118] ^ poly_in[1546:1534];
assign poly_twist_in_1[701:689] = poly_in[701:689] ^ poly_in[1117:1105] ^ poly_in[1533:1521];
assign poly_twist_in_1[688:676] = poly_in[688:676] ^ poly_in[1104:1092] ^ poly_in[1520:1508];
assign poly_twist_in_1[675:663] = poly_in[675:663] ^ poly_in[1091:1079] ^ poly_in[1507:1495];
assign poly_twist_in_1[662:650] = poly_in[662:650] ^ poly_in[1078:1066] ^ poly_in[1494:1482];
assign poly_twist_in_1[649:637] = poly_in[649:637] ^ poly_in[1065:1053] ^ poly_in[1481:1469];
assign poly_twist_in_1[636:624] = poly_in[636:624] ^ poly_in[1052:1040] ^ poly_in[1468:1456];
assign poly_twist_in_1[623:611] = poly_in[623:611] ^ poly_in[1039:1027] ^ poly_in[1455:1443];
assign poly_twist_in_1[610:598] = poly_in[610:598] ^ poly_in[1026:1014] ^ poly_in[1442:1430];
assign poly_twist_in_1[597:585] = poly_in[597:585] ^ poly_in[1013:1001] ^ poly_in[1429:1417];
assign poly_twist_in_1[584:572] = poly_in[584:572] ^ poly_in[1000:988] ^ poly_in[1416:1404];
assign poly_twist_in_1[571:559] = poly_in[571:559] ^ poly_in[987:975] ^ poly_in[1403:1391];
assign poly_twist_in_1[558:546] = poly_in[558:546] ^ poly_in[974:962] ^ poly_in[1390:1378];
assign poly_twist_in_1[545:533] = poly_in[545:533] ^ poly_in[961:949] ^ poly_in[1377:1365];
assign poly_twist_in_1[532:520] = poly_in[532:520] ^ poly_in[948:936] ^ poly_in[1364:1352];
assign poly_twist_in_1[519:507] = poly_in[519:507] ^ poly_in[935:923] ^ poly_in[1351:1339];
assign poly_twist_in_1[506:494] = poly_in[506:494] ^ poly_in[922:910] ^ poly_in[1338:1326];
assign poly_twist_in_1[493:481] = poly_in[493:481] ^ poly_in[909:897] ^ poly_in[1325:1313];
assign poly_twist_in_1[480:468] = poly_in[480:468] ^ poly_in[896:884] ^ poly_in[1312:1300];
assign poly_twist_in_1[467:455] = poly_in[467:455] ^ poly_in[883:871] ^ poly_in[1299:1287];
assign poly_twist_in_1[454:442] = poly_in[454:442] ^ poly_in[870:858] ^ poly_in[1286:1274];
assign poly_twist_in_1[441:429] = poly_in[441:429] ^ poly_in[857:845] ^ poly_in[1273:1261];
assign poly_twist_in_1[428:416] = poly_in[428:416] ^ poly_in[844:832] ^ poly_in[1260:1248];
assign poly_twist_in_1[415:403] = poly_in[415:403];
assign poly_twist_in_1[402:390] = poly_in[402:390];
assign poly_twist_in_1[389:377] = poly_in[389:377];
assign poly_twist_in_1[376:364] = poly_in[376:364];
assign poly_twist_in_1[363:351] = poly_in[363:351];
assign poly_twist_in_1[350:338] = poly_in[350:338];
assign poly_twist_in_1[337:325] = poly_in[337:325];
assign poly_twist_in_1[324:312] = poly_in[324:312];
assign poly_twist_in_1[311:299] = poly_in[311:299];
assign poly_twist_in_1[298:286] = poly_in[298:286];
assign poly_twist_in_1[285:273] = poly_in[285:273];
assign poly_twist_in_1[272:260] = poly_in[272:260];
assign poly_twist_in_1[259:247] = poly_in[259:247];
assign poly_twist_in_1[246:234] = poly_in[246:234];
assign poly_twist_in_1[233:221] = poly_in[233:221];
assign poly_twist_in_1[220:208] = poly_in[220:208];
assign poly_twist_in_1[207:195] = poly_in[207:195];
assign poly_twist_in_1[194:182] = poly_in[194:182];
assign poly_twist_in_1[181:169] = poly_in[181:169];
assign poly_twist_in_1[168:156] = poly_in[168:156];
assign poly_twist_in_1[155:143] = poly_in[155:143];
assign poly_twist_in_1[142:130] = poly_in[142:130];
assign poly_twist_in_1[129:117] = poly_in[129:117];
assign poly_twist_in_1[116:104] = poly_in[116:104];
assign poly_twist_in_1[103:91] = poly_in[103:91];
assign poly_twist_in_1[90:78] = poly_in[90:78];
assign poly_twist_in_1[77:65] = poly_in[77:65];
assign poly_twist_in_1[64:52] = poly_in[64:52];
assign poly_twist_in_1[51:39] = poly_in[51:39];
assign poly_twist_in_1[38:26] = poly_in[38:26];
assign poly_twist_in_1[25:13] = poly_in[25:13];
assign poly_twist_in_1[12:0] = poly_in[12:0];

// output for the 5_th sub radix conversion routine
assign poly_twist_in_2[1663:1651] = poly_twist_in_1_buffer[1663:1651];
assign poly_twist_in_2[1650:1638] = poly_twist_in_1_buffer[1650:1638];
assign poly_twist_in_2[1637:1625] = poly_twist_in_1_buffer[1637:1625];
assign poly_twist_in_2[1624:1612] = poly_twist_in_1_buffer[1624:1612];
assign poly_twist_in_2[1611:1599] = poly_twist_in_1_buffer[1611:1599];
assign poly_twist_in_2[1598:1586] = poly_twist_in_1_buffer[1598:1586];
assign poly_twist_in_2[1585:1573] = poly_twist_in_1_buffer[1585:1573];
assign poly_twist_in_2[1572:1560] = poly_twist_in_1_buffer[1572:1560];
assign poly_twist_in_2[1559:1547] = poly_twist_in_1_buffer[1559:1547];
assign poly_twist_in_2[1546:1534] = poly_twist_in_1_buffer[1546:1534];
assign poly_twist_in_2[1533:1521] = poly_twist_in_1_buffer[1533:1521];
assign poly_twist_in_2[1520:1508] = poly_twist_in_1_buffer[1520:1508];
assign poly_twist_in_2[1507:1495] = poly_twist_in_1_buffer[1507:1495];
assign poly_twist_in_2[1494:1482] = poly_twist_in_1_buffer[1494:1482];
assign poly_twist_in_2[1481:1469] = poly_twist_in_1_buffer[1481:1469];
assign poly_twist_in_2[1468:1456] = poly_twist_in_1_buffer[1468:1456];
assign poly_twist_in_2[1455:1443] = poly_twist_in_1_buffer[1455:1443] ^ poly_twist_in_1_buffer[1663:1651];
assign poly_twist_in_2[1442:1430] = poly_twist_in_1_buffer[1442:1430] ^ poly_twist_in_1_buffer[1650:1638];
assign poly_twist_in_2[1429:1417] = poly_twist_in_1_buffer[1429:1417] ^ poly_twist_in_1_buffer[1637:1625];
assign poly_twist_in_2[1416:1404] = poly_twist_in_1_buffer[1416:1404] ^ poly_twist_in_1_buffer[1624:1612];
assign poly_twist_in_2[1403:1391] = poly_twist_in_1_buffer[1403:1391] ^ poly_twist_in_1_buffer[1611:1599];
assign poly_twist_in_2[1390:1378] = poly_twist_in_1_buffer[1390:1378] ^ poly_twist_in_1_buffer[1598:1586];
assign poly_twist_in_2[1377:1365] = poly_twist_in_1_buffer[1377:1365] ^ poly_twist_in_1_buffer[1585:1573];
assign poly_twist_in_2[1364:1352] = poly_twist_in_1_buffer[1364:1352] ^ poly_twist_in_1_buffer[1572:1560];
assign poly_twist_in_2[1351:1339] = poly_twist_in_1_buffer[1351:1339] ^ poly_twist_in_1_buffer[1559:1547];
assign poly_twist_in_2[1338:1326] = poly_twist_in_1_buffer[1338:1326] ^ poly_twist_in_1_buffer[1546:1534];
assign poly_twist_in_2[1325:1313] = poly_twist_in_1_buffer[1325:1313] ^ poly_twist_in_1_buffer[1533:1521];
assign poly_twist_in_2[1312:1300] = poly_twist_in_1_buffer[1312:1300] ^ poly_twist_in_1_buffer[1520:1508];
assign poly_twist_in_2[1299:1287] = poly_twist_in_1_buffer[1299:1287] ^ poly_twist_in_1_buffer[1507:1495];
assign poly_twist_in_2[1286:1274] = poly_twist_in_1_buffer[1286:1274] ^ poly_twist_in_1_buffer[1494:1482];
assign poly_twist_in_2[1273:1261] = poly_twist_in_1_buffer[1273:1261] ^ poly_twist_in_1_buffer[1481:1469];
assign poly_twist_in_2[1260:1248] = poly_twist_in_1_buffer[1260:1248] ^ poly_twist_in_1_buffer[1468:1456];
assign poly_twist_in_2[1247:1235] = poly_twist_in_1_buffer[1247:1235] ^ poly_twist_in_1_buffer[1455:1443] ^ poly_twist_in_1_buffer[1663:1651];
assign poly_twist_in_2[1234:1222] = poly_twist_in_1_buffer[1234:1222] ^ poly_twist_in_1_buffer[1442:1430] ^ poly_twist_in_1_buffer[1650:1638];
assign poly_twist_in_2[1221:1209] = poly_twist_in_1_buffer[1221:1209] ^ poly_twist_in_1_buffer[1429:1417] ^ poly_twist_in_1_buffer[1637:1625];
assign poly_twist_in_2[1208:1196] = poly_twist_in_1_buffer[1208:1196] ^ poly_twist_in_1_buffer[1416:1404] ^ poly_twist_in_1_buffer[1624:1612];
assign poly_twist_in_2[1195:1183] = poly_twist_in_1_buffer[1195:1183] ^ poly_twist_in_1_buffer[1403:1391] ^ poly_twist_in_1_buffer[1611:1599];
assign poly_twist_in_2[1182:1170] = poly_twist_in_1_buffer[1182:1170] ^ poly_twist_in_1_buffer[1390:1378] ^ poly_twist_in_1_buffer[1598:1586];
assign poly_twist_in_2[1169:1157] = poly_twist_in_1_buffer[1169:1157] ^ poly_twist_in_1_buffer[1377:1365] ^ poly_twist_in_1_buffer[1585:1573];
assign poly_twist_in_2[1156:1144] = poly_twist_in_1_buffer[1156:1144] ^ poly_twist_in_1_buffer[1364:1352] ^ poly_twist_in_1_buffer[1572:1560];
assign poly_twist_in_2[1143:1131] = poly_twist_in_1_buffer[1143:1131] ^ poly_twist_in_1_buffer[1351:1339] ^ poly_twist_in_1_buffer[1559:1547];
assign poly_twist_in_2[1130:1118] = poly_twist_in_1_buffer[1130:1118] ^ poly_twist_in_1_buffer[1338:1326] ^ poly_twist_in_1_buffer[1546:1534];
assign poly_twist_in_2[1117:1105] = poly_twist_in_1_buffer[1117:1105] ^ poly_twist_in_1_buffer[1325:1313] ^ poly_twist_in_1_buffer[1533:1521];
assign poly_twist_in_2[1104:1092] = poly_twist_in_1_buffer[1104:1092] ^ poly_twist_in_1_buffer[1312:1300] ^ poly_twist_in_1_buffer[1520:1508];
assign poly_twist_in_2[1091:1079] = poly_twist_in_1_buffer[1091:1079] ^ poly_twist_in_1_buffer[1299:1287] ^ poly_twist_in_1_buffer[1507:1495];
assign poly_twist_in_2[1078:1066] = poly_twist_in_1_buffer[1078:1066] ^ poly_twist_in_1_buffer[1286:1274] ^ poly_twist_in_1_buffer[1494:1482];
assign poly_twist_in_2[1065:1053] = poly_twist_in_1_buffer[1065:1053] ^ poly_twist_in_1_buffer[1273:1261] ^ poly_twist_in_1_buffer[1481:1469];
assign poly_twist_in_2[1052:1040] = poly_twist_in_1_buffer[1052:1040] ^ poly_twist_in_1_buffer[1260:1248] ^ poly_twist_in_1_buffer[1468:1456];
assign poly_twist_in_2[1039:1027] = poly_twist_in_1_buffer[1039:1027];
assign poly_twist_in_2[1026:1014] = poly_twist_in_1_buffer[1026:1014];
assign poly_twist_in_2[1013:1001] = poly_twist_in_1_buffer[1013:1001];
assign poly_twist_in_2[1000:988] = poly_twist_in_1_buffer[1000:988];
assign poly_twist_in_2[987:975] = poly_twist_in_1_buffer[987:975];
assign poly_twist_in_2[974:962] = poly_twist_in_1_buffer[974:962];
assign poly_twist_in_2[961:949] = poly_twist_in_1_buffer[961:949];
assign poly_twist_in_2[948:936] = poly_twist_in_1_buffer[948:936];
assign poly_twist_in_2[935:923] = poly_twist_in_1_buffer[935:923];
assign poly_twist_in_2[922:910] = poly_twist_in_1_buffer[922:910];
assign poly_twist_in_2[909:897] = poly_twist_in_1_buffer[909:897];
assign poly_twist_in_2[896:884] = poly_twist_in_1_buffer[896:884];
assign poly_twist_in_2[883:871] = poly_twist_in_1_buffer[883:871];
assign poly_twist_in_2[870:858] = poly_twist_in_1_buffer[870:858];
assign poly_twist_in_2[857:845] = poly_twist_in_1_buffer[857:845];
assign poly_twist_in_2[844:832] = poly_twist_in_1_buffer[844:832];
assign poly_twist_in_2[831:819] = poly_twist_in_1_buffer[831:819];
assign poly_twist_in_2[818:806] = poly_twist_in_1_buffer[818:806];
assign poly_twist_in_2[805:793] = poly_twist_in_1_buffer[805:793];
assign poly_twist_in_2[792:780] = poly_twist_in_1_buffer[792:780];
assign poly_twist_in_2[779:767] = poly_twist_in_1_buffer[779:767];
assign poly_twist_in_2[766:754] = poly_twist_in_1_buffer[766:754];
assign poly_twist_in_2[753:741] = poly_twist_in_1_buffer[753:741];
assign poly_twist_in_2[740:728] = poly_twist_in_1_buffer[740:728];
assign poly_twist_in_2[727:715] = poly_twist_in_1_buffer[727:715];
assign poly_twist_in_2[714:702] = poly_twist_in_1_buffer[714:702];
assign poly_twist_in_2[701:689] = poly_twist_in_1_buffer[701:689];
assign poly_twist_in_2[688:676] = poly_twist_in_1_buffer[688:676];
assign poly_twist_in_2[675:663] = poly_twist_in_1_buffer[675:663];
assign poly_twist_in_2[662:650] = poly_twist_in_1_buffer[662:650];
assign poly_twist_in_2[649:637] = poly_twist_in_1_buffer[649:637];
assign poly_twist_in_2[636:624] = poly_twist_in_1_buffer[636:624];
assign poly_twist_in_2[623:611] = poly_twist_in_1_buffer[623:611] ^ poly_twist_in_1_buffer[831:819];
assign poly_twist_in_2[610:598] = poly_twist_in_1_buffer[610:598] ^ poly_twist_in_1_buffer[818:806];
assign poly_twist_in_2[597:585] = poly_twist_in_1_buffer[597:585] ^ poly_twist_in_1_buffer[805:793];
assign poly_twist_in_2[584:572] = poly_twist_in_1_buffer[584:572] ^ poly_twist_in_1_buffer[792:780];
assign poly_twist_in_2[571:559] = poly_twist_in_1_buffer[571:559] ^ poly_twist_in_1_buffer[779:767];
assign poly_twist_in_2[558:546] = poly_twist_in_1_buffer[558:546] ^ poly_twist_in_1_buffer[766:754];
assign poly_twist_in_2[545:533] = poly_twist_in_1_buffer[545:533] ^ poly_twist_in_1_buffer[753:741];
assign poly_twist_in_2[532:520] = poly_twist_in_1_buffer[532:520] ^ poly_twist_in_1_buffer[740:728];
assign poly_twist_in_2[519:507] = poly_twist_in_1_buffer[519:507] ^ poly_twist_in_1_buffer[727:715];
assign poly_twist_in_2[506:494] = poly_twist_in_1_buffer[506:494] ^ poly_twist_in_1_buffer[714:702];
assign poly_twist_in_2[493:481] = poly_twist_in_1_buffer[493:481] ^ poly_twist_in_1_buffer[701:689];
assign poly_twist_in_2[480:468] = poly_twist_in_1_buffer[480:468] ^ poly_twist_in_1_buffer[688:676];
assign poly_twist_in_2[467:455] = poly_twist_in_1_buffer[467:455] ^ poly_twist_in_1_buffer[675:663];
assign poly_twist_in_2[454:442] = poly_twist_in_1_buffer[454:442] ^ poly_twist_in_1_buffer[662:650];
assign poly_twist_in_2[441:429] = poly_twist_in_1_buffer[441:429] ^ poly_twist_in_1_buffer[649:637];
assign poly_twist_in_2[428:416] = poly_twist_in_1_buffer[428:416] ^ poly_twist_in_1_buffer[636:624];
assign poly_twist_in_2[415:403] = poly_twist_in_1_buffer[415:403] ^ poly_twist_in_1_buffer[623:611] ^ poly_twist_in_1_buffer[831:819];
assign poly_twist_in_2[402:390] = poly_twist_in_1_buffer[402:390] ^ poly_twist_in_1_buffer[610:598] ^ poly_twist_in_1_buffer[818:806];
assign poly_twist_in_2[389:377] = poly_twist_in_1_buffer[389:377] ^ poly_twist_in_1_buffer[597:585] ^ poly_twist_in_1_buffer[805:793];
assign poly_twist_in_2[376:364] = poly_twist_in_1_buffer[376:364] ^ poly_twist_in_1_buffer[584:572] ^ poly_twist_in_1_buffer[792:780];
assign poly_twist_in_2[363:351] = poly_twist_in_1_buffer[363:351] ^ poly_twist_in_1_buffer[571:559] ^ poly_twist_in_1_buffer[779:767];
assign poly_twist_in_2[350:338] = poly_twist_in_1_buffer[350:338] ^ poly_twist_in_1_buffer[558:546] ^ poly_twist_in_1_buffer[766:754];
assign poly_twist_in_2[337:325] = poly_twist_in_1_buffer[337:325] ^ poly_twist_in_1_buffer[545:533] ^ poly_twist_in_1_buffer[753:741];
assign poly_twist_in_2[324:312] = poly_twist_in_1_buffer[324:312] ^ poly_twist_in_1_buffer[532:520] ^ poly_twist_in_1_buffer[740:728];
assign poly_twist_in_2[311:299] = poly_twist_in_1_buffer[311:299] ^ poly_twist_in_1_buffer[519:507] ^ poly_twist_in_1_buffer[727:715];
assign poly_twist_in_2[298:286] = poly_twist_in_1_buffer[298:286] ^ poly_twist_in_1_buffer[506:494] ^ poly_twist_in_1_buffer[714:702];
assign poly_twist_in_2[285:273] = poly_twist_in_1_buffer[285:273] ^ poly_twist_in_1_buffer[493:481] ^ poly_twist_in_1_buffer[701:689];
assign poly_twist_in_2[272:260] = poly_twist_in_1_buffer[272:260] ^ poly_twist_in_1_buffer[480:468] ^ poly_twist_in_1_buffer[688:676];
assign poly_twist_in_2[259:247] = poly_twist_in_1_buffer[259:247] ^ poly_twist_in_1_buffer[467:455] ^ poly_twist_in_1_buffer[675:663];
assign poly_twist_in_2[246:234] = poly_twist_in_1_buffer[246:234] ^ poly_twist_in_1_buffer[454:442] ^ poly_twist_in_1_buffer[662:650];
assign poly_twist_in_2[233:221] = poly_twist_in_1_buffer[233:221] ^ poly_twist_in_1_buffer[441:429] ^ poly_twist_in_1_buffer[649:637];
assign poly_twist_in_2[220:208] = poly_twist_in_1_buffer[220:208] ^ poly_twist_in_1_buffer[428:416] ^ poly_twist_in_1_buffer[636:624];
assign poly_twist_in_2[207:195] = poly_twist_in_1_buffer[207:195];
assign poly_twist_in_2[194:182] = poly_twist_in_1_buffer[194:182];
assign poly_twist_in_2[181:169] = poly_twist_in_1_buffer[181:169];
assign poly_twist_in_2[168:156] = poly_twist_in_1_buffer[168:156];
assign poly_twist_in_2[155:143] = poly_twist_in_1_buffer[155:143];
assign poly_twist_in_2[142:130] = poly_twist_in_1_buffer[142:130];
assign poly_twist_in_2[129:117] = poly_twist_in_1_buffer[129:117];
assign poly_twist_in_2[116:104] = poly_twist_in_1_buffer[116:104];
assign poly_twist_in_2[103:91] = poly_twist_in_1_buffer[103:91];
assign poly_twist_in_2[90:78] = poly_twist_in_1_buffer[90:78];
assign poly_twist_in_2[77:65] = poly_twist_in_1_buffer[77:65];
assign poly_twist_in_2[64:52] = poly_twist_in_1_buffer[64:52];
assign poly_twist_in_2[51:39] = poly_twist_in_1_buffer[51:39];
assign poly_twist_in_2[38:26] = poly_twist_in_1_buffer[38:26];
assign poly_twist_in_2[25:13] = poly_twist_in_1_buffer[25:13];
assign poly_twist_in_2[12:0] = poly_twist_in_1_buffer[12:0];

// output for the 4_th sub radix conversion routine
assign poly_twist_in_3[1663:1651] = poly_twist_in_2[1663:1651];
assign poly_twist_in_3[1650:1638] = poly_twist_in_2[1650:1638];
assign poly_twist_in_3[1637:1625] = poly_twist_in_2[1637:1625];
assign poly_twist_in_3[1624:1612] = poly_twist_in_2[1624:1612];
assign poly_twist_in_3[1611:1599] = poly_twist_in_2[1611:1599];
assign poly_twist_in_3[1598:1586] = poly_twist_in_2[1598:1586];
assign poly_twist_in_3[1585:1573] = poly_twist_in_2[1585:1573];
assign poly_twist_in_3[1572:1560] = poly_twist_in_2[1572:1560];
assign poly_twist_in_3[1559:1547] = poly_twist_in_2[1559:1547] ^ poly_twist_in_2[1663:1651];
assign poly_twist_in_3[1546:1534] = poly_twist_in_2[1546:1534] ^ poly_twist_in_2[1650:1638];
assign poly_twist_in_3[1533:1521] = poly_twist_in_2[1533:1521] ^ poly_twist_in_2[1637:1625];
assign poly_twist_in_3[1520:1508] = poly_twist_in_2[1520:1508] ^ poly_twist_in_2[1624:1612];
assign poly_twist_in_3[1507:1495] = poly_twist_in_2[1507:1495] ^ poly_twist_in_2[1611:1599];
assign poly_twist_in_3[1494:1482] = poly_twist_in_2[1494:1482] ^ poly_twist_in_2[1598:1586];
assign poly_twist_in_3[1481:1469] = poly_twist_in_2[1481:1469] ^ poly_twist_in_2[1585:1573];
assign poly_twist_in_3[1468:1456] = poly_twist_in_2[1468:1456] ^ poly_twist_in_2[1572:1560];
assign poly_twist_in_3[1455:1443] = poly_twist_in_2[1455:1443] ^ poly_twist_in_2[1559:1547] ^ poly_twist_in_2[1663:1651];
assign poly_twist_in_3[1442:1430] = poly_twist_in_2[1442:1430] ^ poly_twist_in_2[1546:1534] ^ poly_twist_in_2[1650:1638];
assign poly_twist_in_3[1429:1417] = poly_twist_in_2[1429:1417] ^ poly_twist_in_2[1533:1521] ^ poly_twist_in_2[1637:1625];
assign poly_twist_in_3[1416:1404] = poly_twist_in_2[1416:1404] ^ poly_twist_in_2[1520:1508] ^ poly_twist_in_2[1624:1612];
assign poly_twist_in_3[1403:1391] = poly_twist_in_2[1403:1391] ^ poly_twist_in_2[1507:1495] ^ poly_twist_in_2[1611:1599];
assign poly_twist_in_3[1390:1378] = poly_twist_in_2[1390:1378] ^ poly_twist_in_2[1494:1482] ^ poly_twist_in_2[1598:1586];
assign poly_twist_in_3[1377:1365] = poly_twist_in_2[1377:1365] ^ poly_twist_in_2[1481:1469] ^ poly_twist_in_2[1585:1573];
assign poly_twist_in_3[1364:1352] = poly_twist_in_2[1364:1352] ^ poly_twist_in_2[1468:1456] ^ poly_twist_in_2[1572:1560];
assign poly_twist_in_3[1351:1339] = poly_twist_in_2[1351:1339];
assign poly_twist_in_3[1338:1326] = poly_twist_in_2[1338:1326];
assign poly_twist_in_3[1325:1313] = poly_twist_in_2[1325:1313];
assign poly_twist_in_3[1312:1300] = poly_twist_in_2[1312:1300];
assign poly_twist_in_3[1299:1287] = poly_twist_in_2[1299:1287];
assign poly_twist_in_3[1286:1274] = poly_twist_in_2[1286:1274];
assign poly_twist_in_3[1273:1261] = poly_twist_in_2[1273:1261];
assign poly_twist_in_3[1260:1248] = poly_twist_in_2[1260:1248];
assign poly_twist_in_3[1247:1235] = poly_twist_in_2[1247:1235];
assign poly_twist_in_3[1234:1222] = poly_twist_in_2[1234:1222];
assign poly_twist_in_3[1221:1209] = poly_twist_in_2[1221:1209];
assign poly_twist_in_3[1208:1196] = poly_twist_in_2[1208:1196];
assign poly_twist_in_3[1195:1183] = poly_twist_in_2[1195:1183];
assign poly_twist_in_3[1182:1170] = poly_twist_in_2[1182:1170];
assign poly_twist_in_3[1169:1157] = poly_twist_in_2[1169:1157];
assign poly_twist_in_3[1156:1144] = poly_twist_in_2[1156:1144];
assign poly_twist_in_3[1143:1131] = poly_twist_in_2[1143:1131] ^ poly_twist_in_2[1247:1235];
assign poly_twist_in_3[1130:1118] = poly_twist_in_2[1130:1118] ^ poly_twist_in_2[1234:1222];
assign poly_twist_in_3[1117:1105] = poly_twist_in_2[1117:1105] ^ poly_twist_in_2[1221:1209];
assign poly_twist_in_3[1104:1092] = poly_twist_in_2[1104:1092] ^ poly_twist_in_2[1208:1196];
assign poly_twist_in_3[1091:1079] = poly_twist_in_2[1091:1079] ^ poly_twist_in_2[1195:1183];
assign poly_twist_in_3[1078:1066] = poly_twist_in_2[1078:1066] ^ poly_twist_in_2[1182:1170];
assign poly_twist_in_3[1065:1053] = poly_twist_in_2[1065:1053] ^ poly_twist_in_2[1169:1157];
assign poly_twist_in_3[1052:1040] = poly_twist_in_2[1052:1040] ^ poly_twist_in_2[1156:1144];
assign poly_twist_in_3[1039:1027] = poly_twist_in_2[1039:1027] ^ poly_twist_in_2[1143:1131] ^ poly_twist_in_2[1247:1235];
assign poly_twist_in_3[1026:1014] = poly_twist_in_2[1026:1014] ^ poly_twist_in_2[1130:1118] ^ poly_twist_in_2[1234:1222];
assign poly_twist_in_3[1013:1001] = poly_twist_in_2[1013:1001] ^ poly_twist_in_2[1117:1105] ^ poly_twist_in_2[1221:1209];
assign poly_twist_in_3[1000:988] = poly_twist_in_2[1000:988] ^ poly_twist_in_2[1104:1092] ^ poly_twist_in_2[1208:1196];
assign poly_twist_in_3[987:975] = poly_twist_in_2[987:975] ^ poly_twist_in_2[1091:1079] ^ poly_twist_in_2[1195:1183];
assign poly_twist_in_3[974:962] = poly_twist_in_2[974:962] ^ poly_twist_in_2[1078:1066] ^ poly_twist_in_2[1182:1170];
assign poly_twist_in_3[961:949] = poly_twist_in_2[961:949] ^ poly_twist_in_2[1065:1053] ^ poly_twist_in_2[1169:1157];
assign poly_twist_in_3[948:936] = poly_twist_in_2[948:936] ^ poly_twist_in_2[1052:1040] ^ poly_twist_in_2[1156:1144];
assign poly_twist_in_3[935:923] = poly_twist_in_2[935:923];
assign poly_twist_in_3[922:910] = poly_twist_in_2[922:910];
assign poly_twist_in_3[909:897] = poly_twist_in_2[909:897];
assign poly_twist_in_3[896:884] = poly_twist_in_2[896:884];
assign poly_twist_in_3[883:871] = poly_twist_in_2[883:871];
assign poly_twist_in_3[870:858] = poly_twist_in_2[870:858];
assign poly_twist_in_3[857:845] = poly_twist_in_2[857:845];
assign poly_twist_in_3[844:832] = poly_twist_in_2[844:832];
assign poly_twist_in_3[831:819] = poly_twist_in_2[831:819];
assign poly_twist_in_3[818:806] = poly_twist_in_2[818:806];
assign poly_twist_in_3[805:793] = poly_twist_in_2[805:793];
assign poly_twist_in_3[792:780] = poly_twist_in_2[792:780];
assign poly_twist_in_3[779:767] = poly_twist_in_2[779:767];
assign poly_twist_in_3[766:754] = poly_twist_in_2[766:754];
assign poly_twist_in_3[753:741] = poly_twist_in_2[753:741];
assign poly_twist_in_3[740:728] = poly_twist_in_2[740:728];
assign poly_twist_in_3[727:715] = poly_twist_in_2[727:715] ^ poly_twist_in_2[831:819];
assign poly_twist_in_3[714:702] = poly_twist_in_2[714:702] ^ poly_twist_in_2[818:806];
assign poly_twist_in_3[701:689] = poly_twist_in_2[701:689] ^ poly_twist_in_2[805:793];
assign poly_twist_in_3[688:676] = poly_twist_in_2[688:676] ^ poly_twist_in_2[792:780];
assign poly_twist_in_3[675:663] = poly_twist_in_2[675:663] ^ poly_twist_in_2[779:767];
assign poly_twist_in_3[662:650] = poly_twist_in_2[662:650] ^ poly_twist_in_2[766:754];
assign poly_twist_in_3[649:637] = poly_twist_in_2[649:637] ^ poly_twist_in_2[753:741];
assign poly_twist_in_3[636:624] = poly_twist_in_2[636:624] ^ poly_twist_in_2[740:728];
assign poly_twist_in_3[623:611] = poly_twist_in_2[623:611] ^ poly_twist_in_2[727:715] ^ poly_twist_in_2[831:819];
assign poly_twist_in_3[610:598] = poly_twist_in_2[610:598] ^ poly_twist_in_2[714:702] ^ poly_twist_in_2[818:806];
assign poly_twist_in_3[597:585] = poly_twist_in_2[597:585] ^ poly_twist_in_2[701:689] ^ poly_twist_in_2[805:793];
assign poly_twist_in_3[584:572] = poly_twist_in_2[584:572] ^ poly_twist_in_2[688:676] ^ poly_twist_in_2[792:780];
assign poly_twist_in_3[571:559] = poly_twist_in_2[571:559] ^ poly_twist_in_2[675:663] ^ poly_twist_in_2[779:767];
assign poly_twist_in_3[558:546] = poly_twist_in_2[558:546] ^ poly_twist_in_2[662:650] ^ poly_twist_in_2[766:754];
assign poly_twist_in_3[545:533] = poly_twist_in_2[545:533] ^ poly_twist_in_2[649:637] ^ poly_twist_in_2[753:741];
assign poly_twist_in_3[532:520] = poly_twist_in_2[532:520] ^ poly_twist_in_2[636:624] ^ poly_twist_in_2[740:728];
assign poly_twist_in_3[519:507] = poly_twist_in_2[519:507];
assign poly_twist_in_3[506:494] = poly_twist_in_2[506:494];
assign poly_twist_in_3[493:481] = poly_twist_in_2[493:481];
assign poly_twist_in_3[480:468] = poly_twist_in_2[480:468];
assign poly_twist_in_3[467:455] = poly_twist_in_2[467:455];
assign poly_twist_in_3[454:442] = poly_twist_in_2[454:442];
assign poly_twist_in_3[441:429] = poly_twist_in_2[441:429];
assign poly_twist_in_3[428:416] = poly_twist_in_2[428:416];
assign poly_twist_in_3[415:403] = poly_twist_in_2[415:403];
assign poly_twist_in_3[402:390] = poly_twist_in_2[402:390];
assign poly_twist_in_3[389:377] = poly_twist_in_2[389:377];
assign poly_twist_in_3[376:364] = poly_twist_in_2[376:364];
assign poly_twist_in_3[363:351] = poly_twist_in_2[363:351];
assign poly_twist_in_3[350:338] = poly_twist_in_2[350:338];
assign poly_twist_in_3[337:325] = poly_twist_in_2[337:325];
assign poly_twist_in_3[324:312] = poly_twist_in_2[324:312];
assign poly_twist_in_3[311:299] = poly_twist_in_2[311:299] ^ poly_twist_in_2[415:403];
assign poly_twist_in_3[298:286] = poly_twist_in_2[298:286] ^ poly_twist_in_2[402:390];
assign poly_twist_in_3[285:273] = poly_twist_in_2[285:273] ^ poly_twist_in_2[389:377];
assign poly_twist_in_3[272:260] = poly_twist_in_2[272:260] ^ poly_twist_in_2[376:364];
assign poly_twist_in_3[259:247] = poly_twist_in_2[259:247] ^ poly_twist_in_2[363:351];
assign poly_twist_in_3[246:234] = poly_twist_in_2[246:234] ^ poly_twist_in_2[350:338];
assign poly_twist_in_3[233:221] = poly_twist_in_2[233:221] ^ poly_twist_in_2[337:325];
assign poly_twist_in_3[220:208] = poly_twist_in_2[220:208] ^ poly_twist_in_2[324:312];
assign poly_twist_in_3[207:195] = poly_twist_in_2[207:195] ^ poly_twist_in_2[311:299] ^ poly_twist_in_2[415:403];
assign poly_twist_in_3[194:182] = poly_twist_in_2[194:182] ^ poly_twist_in_2[298:286] ^ poly_twist_in_2[402:390];
assign poly_twist_in_3[181:169] = poly_twist_in_2[181:169] ^ poly_twist_in_2[285:273] ^ poly_twist_in_2[389:377];
assign poly_twist_in_3[168:156] = poly_twist_in_2[168:156] ^ poly_twist_in_2[272:260] ^ poly_twist_in_2[376:364];
assign poly_twist_in_3[155:143] = poly_twist_in_2[155:143] ^ poly_twist_in_2[259:247] ^ poly_twist_in_2[363:351];
assign poly_twist_in_3[142:130] = poly_twist_in_2[142:130] ^ poly_twist_in_2[246:234] ^ poly_twist_in_2[350:338];
assign poly_twist_in_3[129:117] = poly_twist_in_2[129:117] ^ poly_twist_in_2[233:221] ^ poly_twist_in_2[337:325];
assign poly_twist_in_3[116:104] = poly_twist_in_2[116:104] ^ poly_twist_in_2[220:208] ^ poly_twist_in_2[324:312];
assign poly_twist_in_3[103:91] = poly_twist_in_2[103:91];
assign poly_twist_in_3[90:78] = poly_twist_in_2[90:78];
assign poly_twist_in_3[77:65] = poly_twist_in_2[77:65];
assign poly_twist_in_3[64:52] = poly_twist_in_2[64:52];
assign poly_twist_in_3[51:39] = poly_twist_in_2[51:39];
assign poly_twist_in_3[38:26] = poly_twist_in_2[38:26];
assign poly_twist_in_3[25:13] = poly_twist_in_2[25:13];
assign poly_twist_in_3[12:0] = poly_twist_in_2[12:0];

// output for the 3_th sub radix conversion routine
assign poly_twist_in_4[1663:1651] = poly_twist_in_3_buffer[1663:1651];
assign poly_twist_in_4[1650:1638] = poly_twist_in_3_buffer[1650:1638];
assign poly_twist_in_4[1637:1625] = poly_twist_in_3_buffer[1637:1625];
assign poly_twist_in_4[1624:1612] = poly_twist_in_3_buffer[1624:1612];
assign poly_twist_in_4[1611:1599] = poly_twist_in_3_buffer[1611:1599] ^ poly_twist_in_3_buffer[1663:1651];
assign poly_twist_in_4[1598:1586] = poly_twist_in_3_buffer[1598:1586] ^ poly_twist_in_3_buffer[1650:1638];
assign poly_twist_in_4[1585:1573] = poly_twist_in_3_buffer[1585:1573] ^ poly_twist_in_3_buffer[1637:1625];
assign poly_twist_in_4[1572:1560] = poly_twist_in_3_buffer[1572:1560] ^ poly_twist_in_3_buffer[1624:1612];
assign poly_twist_in_4[1559:1547] = poly_twist_in_3_buffer[1559:1547] ^ poly_twist_in_3_buffer[1611:1599] ^ poly_twist_in_3_buffer[1663:1651];
assign poly_twist_in_4[1546:1534] = poly_twist_in_3_buffer[1546:1534] ^ poly_twist_in_3_buffer[1598:1586] ^ poly_twist_in_3_buffer[1650:1638];
assign poly_twist_in_4[1533:1521] = poly_twist_in_3_buffer[1533:1521] ^ poly_twist_in_3_buffer[1585:1573] ^ poly_twist_in_3_buffer[1637:1625];
assign poly_twist_in_4[1520:1508] = poly_twist_in_3_buffer[1520:1508] ^ poly_twist_in_3_buffer[1572:1560] ^ poly_twist_in_3_buffer[1624:1612];
assign poly_twist_in_4[1507:1495] = poly_twist_in_3_buffer[1507:1495];
assign poly_twist_in_4[1494:1482] = poly_twist_in_3_buffer[1494:1482];
assign poly_twist_in_4[1481:1469] = poly_twist_in_3_buffer[1481:1469];
assign poly_twist_in_4[1468:1456] = poly_twist_in_3_buffer[1468:1456];
assign poly_twist_in_4[1455:1443] = poly_twist_in_3_buffer[1455:1443];
assign poly_twist_in_4[1442:1430] = poly_twist_in_3_buffer[1442:1430];
assign poly_twist_in_4[1429:1417] = poly_twist_in_3_buffer[1429:1417];
assign poly_twist_in_4[1416:1404] = poly_twist_in_3_buffer[1416:1404];
assign poly_twist_in_4[1403:1391] = poly_twist_in_3_buffer[1403:1391] ^ poly_twist_in_3_buffer[1455:1443];
assign poly_twist_in_4[1390:1378] = poly_twist_in_3_buffer[1390:1378] ^ poly_twist_in_3_buffer[1442:1430];
assign poly_twist_in_4[1377:1365] = poly_twist_in_3_buffer[1377:1365] ^ poly_twist_in_3_buffer[1429:1417];
assign poly_twist_in_4[1364:1352] = poly_twist_in_3_buffer[1364:1352] ^ poly_twist_in_3_buffer[1416:1404];
assign poly_twist_in_4[1351:1339] = poly_twist_in_3_buffer[1351:1339] ^ poly_twist_in_3_buffer[1403:1391] ^ poly_twist_in_3_buffer[1455:1443];
assign poly_twist_in_4[1338:1326] = poly_twist_in_3_buffer[1338:1326] ^ poly_twist_in_3_buffer[1390:1378] ^ poly_twist_in_3_buffer[1442:1430];
assign poly_twist_in_4[1325:1313] = poly_twist_in_3_buffer[1325:1313] ^ poly_twist_in_3_buffer[1377:1365] ^ poly_twist_in_3_buffer[1429:1417];
assign poly_twist_in_4[1312:1300] = poly_twist_in_3_buffer[1312:1300] ^ poly_twist_in_3_buffer[1364:1352] ^ poly_twist_in_3_buffer[1416:1404];
assign poly_twist_in_4[1299:1287] = poly_twist_in_3_buffer[1299:1287];
assign poly_twist_in_4[1286:1274] = poly_twist_in_3_buffer[1286:1274];
assign poly_twist_in_4[1273:1261] = poly_twist_in_3_buffer[1273:1261];
assign poly_twist_in_4[1260:1248] = poly_twist_in_3_buffer[1260:1248];
assign poly_twist_in_4[1247:1235] = poly_twist_in_3_buffer[1247:1235];
assign poly_twist_in_4[1234:1222] = poly_twist_in_3_buffer[1234:1222];
assign poly_twist_in_4[1221:1209] = poly_twist_in_3_buffer[1221:1209];
assign poly_twist_in_4[1208:1196] = poly_twist_in_3_buffer[1208:1196];
assign poly_twist_in_4[1195:1183] = poly_twist_in_3_buffer[1195:1183] ^ poly_twist_in_3_buffer[1247:1235];
assign poly_twist_in_4[1182:1170] = poly_twist_in_3_buffer[1182:1170] ^ poly_twist_in_3_buffer[1234:1222];
assign poly_twist_in_4[1169:1157] = poly_twist_in_3_buffer[1169:1157] ^ poly_twist_in_3_buffer[1221:1209];
assign poly_twist_in_4[1156:1144] = poly_twist_in_3_buffer[1156:1144] ^ poly_twist_in_3_buffer[1208:1196];
assign poly_twist_in_4[1143:1131] = poly_twist_in_3_buffer[1143:1131] ^ poly_twist_in_3_buffer[1195:1183] ^ poly_twist_in_3_buffer[1247:1235];
assign poly_twist_in_4[1130:1118] = poly_twist_in_3_buffer[1130:1118] ^ poly_twist_in_3_buffer[1182:1170] ^ poly_twist_in_3_buffer[1234:1222];
assign poly_twist_in_4[1117:1105] = poly_twist_in_3_buffer[1117:1105] ^ poly_twist_in_3_buffer[1169:1157] ^ poly_twist_in_3_buffer[1221:1209];
assign poly_twist_in_4[1104:1092] = poly_twist_in_3_buffer[1104:1092] ^ poly_twist_in_3_buffer[1156:1144] ^ poly_twist_in_3_buffer[1208:1196];
assign poly_twist_in_4[1091:1079] = poly_twist_in_3_buffer[1091:1079];
assign poly_twist_in_4[1078:1066] = poly_twist_in_3_buffer[1078:1066];
assign poly_twist_in_4[1065:1053] = poly_twist_in_3_buffer[1065:1053];
assign poly_twist_in_4[1052:1040] = poly_twist_in_3_buffer[1052:1040];
assign poly_twist_in_4[1039:1027] = poly_twist_in_3_buffer[1039:1027];
assign poly_twist_in_4[1026:1014] = poly_twist_in_3_buffer[1026:1014];
assign poly_twist_in_4[1013:1001] = poly_twist_in_3_buffer[1013:1001];
assign poly_twist_in_4[1000:988] = poly_twist_in_3_buffer[1000:988];
assign poly_twist_in_4[987:975] = poly_twist_in_3_buffer[987:975] ^ poly_twist_in_3_buffer[1039:1027];
assign poly_twist_in_4[974:962] = poly_twist_in_3_buffer[974:962] ^ poly_twist_in_3_buffer[1026:1014];
assign poly_twist_in_4[961:949] = poly_twist_in_3_buffer[961:949] ^ poly_twist_in_3_buffer[1013:1001];
assign poly_twist_in_4[948:936] = poly_twist_in_3_buffer[948:936] ^ poly_twist_in_3_buffer[1000:988];
assign poly_twist_in_4[935:923] = poly_twist_in_3_buffer[935:923] ^ poly_twist_in_3_buffer[987:975] ^ poly_twist_in_3_buffer[1039:1027];
assign poly_twist_in_4[922:910] = poly_twist_in_3_buffer[922:910] ^ poly_twist_in_3_buffer[974:962] ^ poly_twist_in_3_buffer[1026:1014];
assign poly_twist_in_4[909:897] = poly_twist_in_3_buffer[909:897] ^ poly_twist_in_3_buffer[961:949] ^ poly_twist_in_3_buffer[1013:1001];
assign poly_twist_in_4[896:884] = poly_twist_in_3_buffer[896:884] ^ poly_twist_in_3_buffer[948:936] ^ poly_twist_in_3_buffer[1000:988];
assign poly_twist_in_4[883:871] = poly_twist_in_3_buffer[883:871];
assign poly_twist_in_4[870:858] = poly_twist_in_3_buffer[870:858];
assign poly_twist_in_4[857:845] = poly_twist_in_3_buffer[857:845];
assign poly_twist_in_4[844:832] = poly_twist_in_3_buffer[844:832];
assign poly_twist_in_4[831:819] = poly_twist_in_3_buffer[831:819];
assign poly_twist_in_4[818:806] = poly_twist_in_3_buffer[818:806];
assign poly_twist_in_4[805:793] = poly_twist_in_3_buffer[805:793];
assign poly_twist_in_4[792:780] = poly_twist_in_3_buffer[792:780];
assign poly_twist_in_4[779:767] = poly_twist_in_3_buffer[779:767] ^ poly_twist_in_3_buffer[831:819];
assign poly_twist_in_4[766:754] = poly_twist_in_3_buffer[766:754] ^ poly_twist_in_3_buffer[818:806];
assign poly_twist_in_4[753:741] = poly_twist_in_3_buffer[753:741] ^ poly_twist_in_3_buffer[805:793];
assign poly_twist_in_4[740:728] = poly_twist_in_3_buffer[740:728] ^ poly_twist_in_3_buffer[792:780];
assign poly_twist_in_4[727:715] = poly_twist_in_3_buffer[727:715] ^ poly_twist_in_3_buffer[779:767] ^ poly_twist_in_3_buffer[831:819];
assign poly_twist_in_4[714:702] = poly_twist_in_3_buffer[714:702] ^ poly_twist_in_3_buffer[766:754] ^ poly_twist_in_3_buffer[818:806];
assign poly_twist_in_4[701:689] = poly_twist_in_3_buffer[701:689] ^ poly_twist_in_3_buffer[753:741] ^ poly_twist_in_3_buffer[805:793];
assign poly_twist_in_4[688:676] = poly_twist_in_3_buffer[688:676] ^ poly_twist_in_3_buffer[740:728] ^ poly_twist_in_3_buffer[792:780];
assign poly_twist_in_4[675:663] = poly_twist_in_3_buffer[675:663];
assign poly_twist_in_4[662:650] = poly_twist_in_3_buffer[662:650];
assign poly_twist_in_4[649:637] = poly_twist_in_3_buffer[649:637];
assign poly_twist_in_4[636:624] = poly_twist_in_3_buffer[636:624];
assign poly_twist_in_4[623:611] = poly_twist_in_3_buffer[623:611];
assign poly_twist_in_4[610:598] = poly_twist_in_3_buffer[610:598];
assign poly_twist_in_4[597:585] = poly_twist_in_3_buffer[597:585];
assign poly_twist_in_4[584:572] = poly_twist_in_3_buffer[584:572];
assign poly_twist_in_4[571:559] = poly_twist_in_3_buffer[571:559] ^ poly_twist_in_3_buffer[623:611];
assign poly_twist_in_4[558:546] = poly_twist_in_3_buffer[558:546] ^ poly_twist_in_3_buffer[610:598];
assign poly_twist_in_4[545:533] = poly_twist_in_3_buffer[545:533] ^ poly_twist_in_3_buffer[597:585];
assign poly_twist_in_4[532:520] = poly_twist_in_3_buffer[532:520] ^ poly_twist_in_3_buffer[584:572];
assign poly_twist_in_4[519:507] = poly_twist_in_3_buffer[519:507] ^ poly_twist_in_3_buffer[571:559] ^ poly_twist_in_3_buffer[623:611];
assign poly_twist_in_4[506:494] = poly_twist_in_3_buffer[506:494] ^ poly_twist_in_3_buffer[558:546] ^ poly_twist_in_3_buffer[610:598];
assign poly_twist_in_4[493:481] = poly_twist_in_3_buffer[493:481] ^ poly_twist_in_3_buffer[545:533] ^ poly_twist_in_3_buffer[597:585];
assign poly_twist_in_4[480:468] = poly_twist_in_3_buffer[480:468] ^ poly_twist_in_3_buffer[532:520] ^ poly_twist_in_3_buffer[584:572];
assign poly_twist_in_4[467:455] = poly_twist_in_3_buffer[467:455];
assign poly_twist_in_4[454:442] = poly_twist_in_3_buffer[454:442];
assign poly_twist_in_4[441:429] = poly_twist_in_3_buffer[441:429];
assign poly_twist_in_4[428:416] = poly_twist_in_3_buffer[428:416];
assign poly_twist_in_4[415:403] = poly_twist_in_3_buffer[415:403];
assign poly_twist_in_4[402:390] = poly_twist_in_3_buffer[402:390];
assign poly_twist_in_4[389:377] = poly_twist_in_3_buffer[389:377];
assign poly_twist_in_4[376:364] = poly_twist_in_3_buffer[376:364];
assign poly_twist_in_4[363:351] = poly_twist_in_3_buffer[363:351] ^ poly_twist_in_3_buffer[415:403];
assign poly_twist_in_4[350:338] = poly_twist_in_3_buffer[350:338] ^ poly_twist_in_3_buffer[402:390];
assign poly_twist_in_4[337:325] = poly_twist_in_3_buffer[337:325] ^ poly_twist_in_3_buffer[389:377];
assign poly_twist_in_4[324:312] = poly_twist_in_3_buffer[324:312] ^ poly_twist_in_3_buffer[376:364];
assign poly_twist_in_4[311:299] = poly_twist_in_3_buffer[311:299] ^ poly_twist_in_3_buffer[363:351] ^ poly_twist_in_3_buffer[415:403];
assign poly_twist_in_4[298:286] = poly_twist_in_3_buffer[298:286] ^ poly_twist_in_3_buffer[350:338] ^ poly_twist_in_3_buffer[402:390];
assign poly_twist_in_4[285:273] = poly_twist_in_3_buffer[285:273] ^ poly_twist_in_3_buffer[337:325] ^ poly_twist_in_3_buffer[389:377];
assign poly_twist_in_4[272:260] = poly_twist_in_3_buffer[272:260] ^ poly_twist_in_3_buffer[324:312] ^ poly_twist_in_3_buffer[376:364];
assign poly_twist_in_4[259:247] = poly_twist_in_3_buffer[259:247];
assign poly_twist_in_4[246:234] = poly_twist_in_3_buffer[246:234];
assign poly_twist_in_4[233:221] = poly_twist_in_3_buffer[233:221];
assign poly_twist_in_4[220:208] = poly_twist_in_3_buffer[220:208];
assign poly_twist_in_4[207:195] = poly_twist_in_3_buffer[207:195];
assign poly_twist_in_4[194:182] = poly_twist_in_3_buffer[194:182];
assign poly_twist_in_4[181:169] = poly_twist_in_3_buffer[181:169];
assign poly_twist_in_4[168:156] = poly_twist_in_3_buffer[168:156];
assign poly_twist_in_4[155:143] = poly_twist_in_3_buffer[155:143] ^ poly_twist_in_3_buffer[207:195];
assign poly_twist_in_4[142:130] = poly_twist_in_3_buffer[142:130] ^ poly_twist_in_3_buffer[194:182];
assign poly_twist_in_4[129:117] = poly_twist_in_3_buffer[129:117] ^ poly_twist_in_3_buffer[181:169];
assign poly_twist_in_4[116:104] = poly_twist_in_3_buffer[116:104] ^ poly_twist_in_3_buffer[168:156];
assign poly_twist_in_4[103:91] = poly_twist_in_3_buffer[103:91] ^ poly_twist_in_3_buffer[155:143] ^ poly_twist_in_3_buffer[207:195];
assign poly_twist_in_4[90:78] = poly_twist_in_3_buffer[90:78] ^ poly_twist_in_3_buffer[142:130] ^ poly_twist_in_3_buffer[194:182];
assign poly_twist_in_4[77:65] = poly_twist_in_3_buffer[77:65] ^ poly_twist_in_3_buffer[129:117] ^ poly_twist_in_3_buffer[181:169];
assign poly_twist_in_4[64:52] = poly_twist_in_3_buffer[64:52] ^ poly_twist_in_3_buffer[116:104] ^ poly_twist_in_3_buffer[168:156];
assign poly_twist_in_4[51:39] = poly_twist_in_3_buffer[51:39];
assign poly_twist_in_4[38:26] = poly_twist_in_3_buffer[38:26];
assign poly_twist_in_4[25:13] = poly_twist_in_3_buffer[25:13];
assign poly_twist_in_4[12:0] = poly_twist_in_3_buffer[12:0];

// output for the 2_th sub radix conversion routine
assign poly_twist_in_5[1663:1651] = poly_twist_in_4[1663:1651];
assign poly_twist_in_5[1650:1638] = poly_twist_in_4[1650:1638];
assign poly_twist_in_5[1637:1625] = poly_twist_in_4[1637:1625] ^ poly_twist_in_4[1663:1651];
assign poly_twist_in_5[1624:1612] = poly_twist_in_4[1624:1612] ^ poly_twist_in_4[1650:1638];
assign poly_twist_in_5[1611:1599] = poly_twist_in_4[1611:1599] ^ poly_twist_in_4[1637:1625] ^ poly_twist_in_4[1663:1651];
assign poly_twist_in_5[1598:1586] = poly_twist_in_4[1598:1586] ^ poly_twist_in_4[1624:1612] ^ poly_twist_in_4[1650:1638];
assign poly_twist_in_5[1585:1573] = poly_twist_in_4[1585:1573];
assign poly_twist_in_5[1572:1560] = poly_twist_in_4[1572:1560];
assign poly_twist_in_5[1559:1547] = poly_twist_in_4[1559:1547];
assign poly_twist_in_5[1546:1534] = poly_twist_in_4[1546:1534];
assign poly_twist_in_5[1533:1521] = poly_twist_in_4[1533:1521] ^ poly_twist_in_4[1559:1547];
assign poly_twist_in_5[1520:1508] = poly_twist_in_4[1520:1508] ^ poly_twist_in_4[1546:1534];
assign poly_twist_in_5[1507:1495] = poly_twist_in_4[1507:1495] ^ poly_twist_in_4[1533:1521] ^ poly_twist_in_4[1559:1547];
assign poly_twist_in_5[1494:1482] = poly_twist_in_4[1494:1482] ^ poly_twist_in_4[1520:1508] ^ poly_twist_in_4[1546:1534];
assign poly_twist_in_5[1481:1469] = poly_twist_in_4[1481:1469];
assign poly_twist_in_5[1468:1456] = poly_twist_in_4[1468:1456];
assign poly_twist_in_5[1455:1443] = poly_twist_in_4[1455:1443];
assign poly_twist_in_5[1442:1430] = poly_twist_in_4[1442:1430];
assign poly_twist_in_5[1429:1417] = poly_twist_in_4[1429:1417] ^ poly_twist_in_4[1455:1443];
assign poly_twist_in_5[1416:1404] = poly_twist_in_4[1416:1404] ^ poly_twist_in_4[1442:1430];
assign poly_twist_in_5[1403:1391] = poly_twist_in_4[1403:1391] ^ poly_twist_in_4[1429:1417] ^ poly_twist_in_4[1455:1443];
assign poly_twist_in_5[1390:1378] = poly_twist_in_4[1390:1378] ^ poly_twist_in_4[1416:1404] ^ poly_twist_in_4[1442:1430];
assign poly_twist_in_5[1377:1365] = poly_twist_in_4[1377:1365];
assign poly_twist_in_5[1364:1352] = poly_twist_in_4[1364:1352];
assign poly_twist_in_5[1351:1339] = poly_twist_in_4[1351:1339];
assign poly_twist_in_5[1338:1326] = poly_twist_in_4[1338:1326];
assign poly_twist_in_5[1325:1313] = poly_twist_in_4[1325:1313] ^ poly_twist_in_4[1351:1339];
assign poly_twist_in_5[1312:1300] = poly_twist_in_4[1312:1300] ^ poly_twist_in_4[1338:1326];
assign poly_twist_in_5[1299:1287] = poly_twist_in_4[1299:1287] ^ poly_twist_in_4[1325:1313] ^ poly_twist_in_4[1351:1339];
assign poly_twist_in_5[1286:1274] = poly_twist_in_4[1286:1274] ^ poly_twist_in_4[1312:1300] ^ poly_twist_in_4[1338:1326];
assign poly_twist_in_5[1273:1261] = poly_twist_in_4[1273:1261];
assign poly_twist_in_5[1260:1248] = poly_twist_in_4[1260:1248];
assign poly_twist_in_5[1247:1235] = poly_twist_in_4[1247:1235];
assign poly_twist_in_5[1234:1222] = poly_twist_in_4[1234:1222];
assign poly_twist_in_5[1221:1209] = poly_twist_in_4[1221:1209] ^ poly_twist_in_4[1247:1235];
assign poly_twist_in_5[1208:1196] = poly_twist_in_4[1208:1196] ^ poly_twist_in_4[1234:1222];
assign poly_twist_in_5[1195:1183] = poly_twist_in_4[1195:1183] ^ poly_twist_in_4[1221:1209] ^ poly_twist_in_4[1247:1235];
assign poly_twist_in_5[1182:1170] = poly_twist_in_4[1182:1170] ^ poly_twist_in_4[1208:1196] ^ poly_twist_in_4[1234:1222];
assign poly_twist_in_5[1169:1157] = poly_twist_in_4[1169:1157];
assign poly_twist_in_5[1156:1144] = poly_twist_in_4[1156:1144];
assign poly_twist_in_5[1143:1131] = poly_twist_in_4[1143:1131];
assign poly_twist_in_5[1130:1118] = poly_twist_in_4[1130:1118];
assign poly_twist_in_5[1117:1105] = poly_twist_in_4[1117:1105] ^ poly_twist_in_4[1143:1131];
assign poly_twist_in_5[1104:1092] = poly_twist_in_4[1104:1092] ^ poly_twist_in_4[1130:1118];
assign poly_twist_in_5[1091:1079] = poly_twist_in_4[1091:1079] ^ poly_twist_in_4[1117:1105] ^ poly_twist_in_4[1143:1131];
assign poly_twist_in_5[1078:1066] = poly_twist_in_4[1078:1066] ^ poly_twist_in_4[1104:1092] ^ poly_twist_in_4[1130:1118];
assign poly_twist_in_5[1065:1053] = poly_twist_in_4[1065:1053];
assign poly_twist_in_5[1052:1040] = poly_twist_in_4[1052:1040];
assign poly_twist_in_5[1039:1027] = poly_twist_in_4[1039:1027];
assign poly_twist_in_5[1026:1014] = poly_twist_in_4[1026:1014];
assign poly_twist_in_5[1013:1001] = poly_twist_in_4[1013:1001] ^ poly_twist_in_4[1039:1027];
assign poly_twist_in_5[1000:988] = poly_twist_in_4[1000:988] ^ poly_twist_in_4[1026:1014];
assign poly_twist_in_5[987:975] = poly_twist_in_4[987:975] ^ poly_twist_in_4[1013:1001] ^ poly_twist_in_4[1039:1027];
assign poly_twist_in_5[974:962] = poly_twist_in_4[974:962] ^ poly_twist_in_4[1000:988] ^ poly_twist_in_4[1026:1014];
assign poly_twist_in_5[961:949] = poly_twist_in_4[961:949];
assign poly_twist_in_5[948:936] = poly_twist_in_4[948:936];
assign poly_twist_in_5[935:923] = poly_twist_in_4[935:923];
assign poly_twist_in_5[922:910] = poly_twist_in_4[922:910];
assign poly_twist_in_5[909:897] = poly_twist_in_4[909:897] ^ poly_twist_in_4[935:923];
assign poly_twist_in_5[896:884] = poly_twist_in_4[896:884] ^ poly_twist_in_4[922:910];
assign poly_twist_in_5[883:871] = poly_twist_in_4[883:871] ^ poly_twist_in_4[909:897] ^ poly_twist_in_4[935:923];
assign poly_twist_in_5[870:858] = poly_twist_in_4[870:858] ^ poly_twist_in_4[896:884] ^ poly_twist_in_4[922:910];
assign poly_twist_in_5[857:845] = poly_twist_in_4[857:845];
assign poly_twist_in_5[844:832] = poly_twist_in_4[844:832];
assign poly_twist_in_5[831:819] = poly_twist_in_4[831:819];
assign poly_twist_in_5[818:806] = poly_twist_in_4[818:806];
assign poly_twist_in_5[805:793] = poly_twist_in_4[805:793] ^ poly_twist_in_4[831:819];
assign poly_twist_in_5[792:780] = poly_twist_in_4[792:780] ^ poly_twist_in_4[818:806];
assign poly_twist_in_5[779:767] = poly_twist_in_4[779:767] ^ poly_twist_in_4[805:793] ^ poly_twist_in_4[831:819];
assign poly_twist_in_5[766:754] = poly_twist_in_4[766:754] ^ poly_twist_in_4[792:780] ^ poly_twist_in_4[818:806];
assign poly_twist_in_5[753:741] = poly_twist_in_4[753:741];
assign poly_twist_in_5[740:728] = poly_twist_in_4[740:728];
assign poly_twist_in_5[727:715] = poly_twist_in_4[727:715];
assign poly_twist_in_5[714:702] = poly_twist_in_4[714:702];
assign poly_twist_in_5[701:689] = poly_twist_in_4[701:689] ^ poly_twist_in_4[727:715];
assign poly_twist_in_5[688:676] = poly_twist_in_4[688:676] ^ poly_twist_in_4[714:702];
assign poly_twist_in_5[675:663] = poly_twist_in_4[675:663] ^ poly_twist_in_4[701:689] ^ poly_twist_in_4[727:715];
assign poly_twist_in_5[662:650] = poly_twist_in_4[662:650] ^ poly_twist_in_4[688:676] ^ poly_twist_in_4[714:702];
assign poly_twist_in_5[649:637] = poly_twist_in_4[649:637];
assign poly_twist_in_5[636:624] = poly_twist_in_4[636:624];
assign poly_twist_in_5[623:611] = poly_twist_in_4[623:611];
assign poly_twist_in_5[610:598] = poly_twist_in_4[610:598];
assign poly_twist_in_5[597:585] = poly_twist_in_4[597:585] ^ poly_twist_in_4[623:611];
assign poly_twist_in_5[584:572] = poly_twist_in_4[584:572] ^ poly_twist_in_4[610:598];
assign poly_twist_in_5[571:559] = poly_twist_in_4[571:559] ^ poly_twist_in_4[597:585] ^ poly_twist_in_4[623:611];
assign poly_twist_in_5[558:546] = poly_twist_in_4[558:546] ^ poly_twist_in_4[584:572] ^ poly_twist_in_4[610:598];
assign poly_twist_in_5[545:533] = poly_twist_in_4[545:533];
assign poly_twist_in_5[532:520] = poly_twist_in_4[532:520];
assign poly_twist_in_5[519:507] = poly_twist_in_4[519:507];
assign poly_twist_in_5[506:494] = poly_twist_in_4[506:494];
assign poly_twist_in_5[493:481] = poly_twist_in_4[493:481] ^ poly_twist_in_4[519:507];
assign poly_twist_in_5[480:468] = poly_twist_in_4[480:468] ^ poly_twist_in_4[506:494];
assign poly_twist_in_5[467:455] = poly_twist_in_4[467:455] ^ poly_twist_in_4[493:481] ^ poly_twist_in_4[519:507];
assign poly_twist_in_5[454:442] = poly_twist_in_4[454:442] ^ poly_twist_in_4[480:468] ^ poly_twist_in_4[506:494];
assign poly_twist_in_5[441:429] = poly_twist_in_4[441:429];
assign poly_twist_in_5[428:416] = poly_twist_in_4[428:416];
assign poly_twist_in_5[415:403] = poly_twist_in_4[415:403];
assign poly_twist_in_5[402:390] = poly_twist_in_4[402:390];
assign poly_twist_in_5[389:377] = poly_twist_in_4[389:377] ^ poly_twist_in_4[415:403];
assign poly_twist_in_5[376:364] = poly_twist_in_4[376:364] ^ poly_twist_in_4[402:390];
assign poly_twist_in_5[363:351] = poly_twist_in_4[363:351] ^ poly_twist_in_4[389:377] ^ poly_twist_in_4[415:403];
assign poly_twist_in_5[350:338] = poly_twist_in_4[350:338] ^ poly_twist_in_4[376:364] ^ poly_twist_in_4[402:390];
assign poly_twist_in_5[337:325] = poly_twist_in_4[337:325];
assign poly_twist_in_5[324:312] = poly_twist_in_4[324:312];
assign poly_twist_in_5[311:299] = poly_twist_in_4[311:299];
assign poly_twist_in_5[298:286] = poly_twist_in_4[298:286];
assign poly_twist_in_5[285:273] = poly_twist_in_4[285:273] ^ poly_twist_in_4[311:299];
assign poly_twist_in_5[272:260] = poly_twist_in_4[272:260] ^ poly_twist_in_4[298:286];
assign poly_twist_in_5[259:247] = poly_twist_in_4[259:247] ^ poly_twist_in_4[285:273] ^ poly_twist_in_4[311:299];
assign poly_twist_in_5[246:234] = poly_twist_in_4[246:234] ^ poly_twist_in_4[272:260] ^ poly_twist_in_4[298:286];
assign poly_twist_in_5[233:221] = poly_twist_in_4[233:221];
assign poly_twist_in_5[220:208] = poly_twist_in_4[220:208];
assign poly_twist_in_5[207:195] = poly_twist_in_4[207:195];
assign poly_twist_in_5[194:182] = poly_twist_in_4[194:182];
assign poly_twist_in_5[181:169] = poly_twist_in_4[181:169] ^ poly_twist_in_4[207:195];
assign poly_twist_in_5[168:156] = poly_twist_in_4[168:156] ^ poly_twist_in_4[194:182];
assign poly_twist_in_5[155:143] = poly_twist_in_4[155:143] ^ poly_twist_in_4[181:169] ^ poly_twist_in_4[207:195];
assign poly_twist_in_5[142:130] = poly_twist_in_4[142:130] ^ poly_twist_in_4[168:156] ^ poly_twist_in_4[194:182];
assign poly_twist_in_5[129:117] = poly_twist_in_4[129:117];
assign poly_twist_in_5[116:104] = poly_twist_in_4[116:104];
assign poly_twist_in_5[103:91] = poly_twist_in_4[103:91];
assign poly_twist_in_5[90:78] = poly_twist_in_4[90:78];
assign poly_twist_in_5[77:65] = poly_twist_in_4[77:65] ^ poly_twist_in_4[103:91];
assign poly_twist_in_5[64:52] = poly_twist_in_4[64:52] ^ poly_twist_in_4[90:78];
assign poly_twist_in_5[51:39] = poly_twist_in_4[51:39] ^ poly_twist_in_4[77:65] ^ poly_twist_in_4[103:91];
assign poly_twist_in_5[38:26] = poly_twist_in_4[38:26] ^ poly_twist_in_4[64:52] ^ poly_twist_in_4[90:78];
assign poly_twist_in_5[25:13] = poly_twist_in_4[25:13];
assign poly_twist_in_5[12:0] = poly_twist_in_4[12:0];

// output for the 1_th sub radix conversion routine
assign poly_twist_in_6[1663:1651] = poly_twist_in_5_buffer[1663:1651];
assign poly_twist_in_6[1650:1638] = poly_twist_in_5_buffer[1650:1638] ^ poly_twist_in_5_buffer[1663:1651];
assign poly_twist_in_6[1637:1625] = poly_twist_in_5_buffer[1637:1625] ^ poly_twist_in_5_buffer[1650:1638] ^ poly_twist_in_5_buffer[1663:1651];
assign poly_twist_in_6[1624:1612] = poly_twist_in_5_buffer[1624:1612];
assign poly_twist_in_6[1611:1599] = poly_twist_in_5_buffer[1611:1599];
assign poly_twist_in_6[1598:1586] = poly_twist_in_5_buffer[1598:1586] ^ poly_twist_in_5_buffer[1611:1599];
assign poly_twist_in_6[1585:1573] = poly_twist_in_5_buffer[1585:1573] ^ poly_twist_in_5_buffer[1598:1586] ^ poly_twist_in_5_buffer[1611:1599];
assign poly_twist_in_6[1572:1560] = poly_twist_in_5_buffer[1572:1560];
assign poly_twist_in_6[1559:1547] = poly_twist_in_5_buffer[1559:1547];
assign poly_twist_in_6[1546:1534] = poly_twist_in_5_buffer[1546:1534] ^ poly_twist_in_5_buffer[1559:1547];
assign poly_twist_in_6[1533:1521] = poly_twist_in_5_buffer[1533:1521] ^ poly_twist_in_5_buffer[1546:1534] ^ poly_twist_in_5_buffer[1559:1547];
assign poly_twist_in_6[1520:1508] = poly_twist_in_5_buffer[1520:1508];
assign poly_twist_in_6[1507:1495] = poly_twist_in_5_buffer[1507:1495];
assign poly_twist_in_6[1494:1482] = poly_twist_in_5_buffer[1494:1482] ^ poly_twist_in_5_buffer[1507:1495];
assign poly_twist_in_6[1481:1469] = poly_twist_in_5_buffer[1481:1469] ^ poly_twist_in_5_buffer[1494:1482] ^ poly_twist_in_5_buffer[1507:1495];
assign poly_twist_in_6[1468:1456] = poly_twist_in_5_buffer[1468:1456];
assign poly_twist_in_6[1455:1443] = poly_twist_in_5_buffer[1455:1443];
assign poly_twist_in_6[1442:1430] = poly_twist_in_5_buffer[1442:1430] ^ poly_twist_in_5_buffer[1455:1443];
assign poly_twist_in_6[1429:1417] = poly_twist_in_5_buffer[1429:1417] ^ poly_twist_in_5_buffer[1442:1430] ^ poly_twist_in_5_buffer[1455:1443];
assign poly_twist_in_6[1416:1404] = poly_twist_in_5_buffer[1416:1404];
assign poly_twist_in_6[1403:1391] = poly_twist_in_5_buffer[1403:1391];
assign poly_twist_in_6[1390:1378] = poly_twist_in_5_buffer[1390:1378] ^ poly_twist_in_5_buffer[1403:1391];
assign poly_twist_in_6[1377:1365] = poly_twist_in_5_buffer[1377:1365] ^ poly_twist_in_5_buffer[1390:1378] ^ poly_twist_in_5_buffer[1403:1391];
assign poly_twist_in_6[1364:1352] = poly_twist_in_5_buffer[1364:1352];
assign poly_twist_in_6[1351:1339] = poly_twist_in_5_buffer[1351:1339];
assign poly_twist_in_6[1338:1326] = poly_twist_in_5_buffer[1338:1326] ^ poly_twist_in_5_buffer[1351:1339];
assign poly_twist_in_6[1325:1313] = poly_twist_in_5_buffer[1325:1313] ^ poly_twist_in_5_buffer[1338:1326] ^ poly_twist_in_5_buffer[1351:1339];
assign poly_twist_in_6[1312:1300] = poly_twist_in_5_buffer[1312:1300];
assign poly_twist_in_6[1299:1287] = poly_twist_in_5_buffer[1299:1287];
assign poly_twist_in_6[1286:1274] = poly_twist_in_5_buffer[1286:1274] ^ poly_twist_in_5_buffer[1299:1287];
assign poly_twist_in_6[1273:1261] = poly_twist_in_5_buffer[1273:1261] ^ poly_twist_in_5_buffer[1286:1274] ^ poly_twist_in_5_buffer[1299:1287];
assign poly_twist_in_6[1260:1248] = poly_twist_in_5_buffer[1260:1248];
assign poly_twist_in_6[1247:1235] = poly_twist_in_5_buffer[1247:1235];
assign poly_twist_in_6[1234:1222] = poly_twist_in_5_buffer[1234:1222] ^ poly_twist_in_5_buffer[1247:1235];
assign poly_twist_in_6[1221:1209] = poly_twist_in_5_buffer[1221:1209] ^ poly_twist_in_5_buffer[1234:1222] ^ poly_twist_in_5_buffer[1247:1235];
assign poly_twist_in_6[1208:1196] = poly_twist_in_5_buffer[1208:1196];
assign poly_twist_in_6[1195:1183] = poly_twist_in_5_buffer[1195:1183];
assign poly_twist_in_6[1182:1170] = poly_twist_in_5_buffer[1182:1170] ^ poly_twist_in_5_buffer[1195:1183];
assign poly_twist_in_6[1169:1157] = poly_twist_in_5_buffer[1169:1157] ^ poly_twist_in_5_buffer[1182:1170] ^ poly_twist_in_5_buffer[1195:1183];
assign poly_twist_in_6[1156:1144] = poly_twist_in_5_buffer[1156:1144];
assign poly_twist_in_6[1143:1131] = poly_twist_in_5_buffer[1143:1131];
assign poly_twist_in_6[1130:1118] = poly_twist_in_5_buffer[1130:1118] ^ poly_twist_in_5_buffer[1143:1131];
assign poly_twist_in_6[1117:1105] = poly_twist_in_5_buffer[1117:1105] ^ poly_twist_in_5_buffer[1130:1118] ^ poly_twist_in_5_buffer[1143:1131];
assign poly_twist_in_6[1104:1092] = poly_twist_in_5_buffer[1104:1092];
assign poly_twist_in_6[1091:1079] = poly_twist_in_5_buffer[1091:1079];
assign poly_twist_in_6[1078:1066] = poly_twist_in_5_buffer[1078:1066] ^ poly_twist_in_5_buffer[1091:1079];
assign poly_twist_in_6[1065:1053] = poly_twist_in_5_buffer[1065:1053] ^ poly_twist_in_5_buffer[1078:1066] ^ poly_twist_in_5_buffer[1091:1079];
assign poly_twist_in_6[1052:1040] = poly_twist_in_5_buffer[1052:1040];
assign poly_twist_in_6[1039:1027] = poly_twist_in_5_buffer[1039:1027];
assign poly_twist_in_6[1026:1014] = poly_twist_in_5_buffer[1026:1014] ^ poly_twist_in_5_buffer[1039:1027];
assign poly_twist_in_6[1013:1001] = poly_twist_in_5_buffer[1013:1001] ^ poly_twist_in_5_buffer[1026:1014] ^ poly_twist_in_5_buffer[1039:1027];
assign poly_twist_in_6[1000:988] = poly_twist_in_5_buffer[1000:988];
assign poly_twist_in_6[987:975] = poly_twist_in_5_buffer[987:975];
assign poly_twist_in_6[974:962] = poly_twist_in_5_buffer[974:962] ^ poly_twist_in_5_buffer[987:975];
assign poly_twist_in_6[961:949] = poly_twist_in_5_buffer[961:949] ^ poly_twist_in_5_buffer[974:962] ^ poly_twist_in_5_buffer[987:975];
assign poly_twist_in_6[948:936] = poly_twist_in_5_buffer[948:936];
assign poly_twist_in_6[935:923] = poly_twist_in_5_buffer[935:923];
assign poly_twist_in_6[922:910] = poly_twist_in_5_buffer[922:910] ^ poly_twist_in_5_buffer[935:923];
assign poly_twist_in_6[909:897] = poly_twist_in_5_buffer[909:897] ^ poly_twist_in_5_buffer[922:910] ^ poly_twist_in_5_buffer[935:923];
assign poly_twist_in_6[896:884] = poly_twist_in_5_buffer[896:884];
assign poly_twist_in_6[883:871] = poly_twist_in_5_buffer[883:871];
assign poly_twist_in_6[870:858] = poly_twist_in_5_buffer[870:858] ^ poly_twist_in_5_buffer[883:871];
assign poly_twist_in_6[857:845] = poly_twist_in_5_buffer[857:845] ^ poly_twist_in_5_buffer[870:858] ^ poly_twist_in_5_buffer[883:871];
assign poly_twist_in_6[844:832] = poly_twist_in_5_buffer[844:832];
assign poly_twist_in_6[831:819] = poly_twist_in_5_buffer[831:819];
assign poly_twist_in_6[818:806] = poly_twist_in_5_buffer[818:806] ^ poly_twist_in_5_buffer[831:819];
assign poly_twist_in_6[805:793] = poly_twist_in_5_buffer[805:793] ^ poly_twist_in_5_buffer[818:806] ^ poly_twist_in_5_buffer[831:819];
assign poly_twist_in_6[792:780] = poly_twist_in_5_buffer[792:780];
assign poly_twist_in_6[779:767] = poly_twist_in_5_buffer[779:767];
assign poly_twist_in_6[766:754] = poly_twist_in_5_buffer[766:754] ^ poly_twist_in_5_buffer[779:767];
assign poly_twist_in_6[753:741] = poly_twist_in_5_buffer[753:741] ^ poly_twist_in_5_buffer[766:754] ^ poly_twist_in_5_buffer[779:767];
assign poly_twist_in_6[740:728] = poly_twist_in_5_buffer[740:728];
assign poly_twist_in_6[727:715] = poly_twist_in_5_buffer[727:715];
assign poly_twist_in_6[714:702] = poly_twist_in_5_buffer[714:702] ^ poly_twist_in_5_buffer[727:715];
assign poly_twist_in_6[701:689] = poly_twist_in_5_buffer[701:689] ^ poly_twist_in_5_buffer[714:702] ^ poly_twist_in_5_buffer[727:715];
assign poly_twist_in_6[688:676] = poly_twist_in_5_buffer[688:676];
assign poly_twist_in_6[675:663] = poly_twist_in_5_buffer[675:663];
assign poly_twist_in_6[662:650] = poly_twist_in_5_buffer[662:650] ^ poly_twist_in_5_buffer[675:663];
assign poly_twist_in_6[649:637] = poly_twist_in_5_buffer[649:637] ^ poly_twist_in_5_buffer[662:650] ^ poly_twist_in_5_buffer[675:663];
assign poly_twist_in_6[636:624] = poly_twist_in_5_buffer[636:624];
assign poly_twist_in_6[623:611] = poly_twist_in_5_buffer[623:611];
assign poly_twist_in_6[610:598] = poly_twist_in_5_buffer[610:598] ^ poly_twist_in_5_buffer[623:611];
assign poly_twist_in_6[597:585] = poly_twist_in_5_buffer[597:585] ^ poly_twist_in_5_buffer[610:598] ^ poly_twist_in_5_buffer[623:611];
assign poly_twist_in_6[584:572] = poly_twist_in_5_buffer[584:572];
assign poly_twist_in_6[571:559] = poly_twist_in_5_buffer[571:559];
assign poly_twist_in_6[558:546] = poly_twist_in_5_buffer[558:546] ^ poly_twist_in_5_buffer[571:559];
assign poly_twist_in_6[545:533] = poly_twist_in_5_buffer[545:533] ^ poly_twist_in_5_buffer[558:546] ^ poly_twist_in_5_buffer[571:559];
assign poly_twist_in_6[532:520] = poly_twist_in_5_buffer[532:520];
assign poly_twist_in_6[519:507] = poly_twist_in_5_buffer[519:507];
assign poly_twist_in_6[506:494] = poly_twist_in_5_buffer[506:494] ^ poly_twist_in_5_buffer[519:507];
assign poly_twist_in_6[493:481] = poly_twist_in_5_buffer[493:481] ^ poly_twist_in_5_buffer[506:494] ^ poly_twist_in_5_buffer[519:507];
assign poly_twist_in_6[480:468] = poly_twist_in_5_buffer[480:468];
assign poly_twist_in_6[467:455] = poly_twist_in_5_buffer[467:455];
assign poly_twist_in_6[454:442] = poly_twist_in_5_buffer[454:442] ^ poly_twist_in_5_buffer[467:455];
assign poly_twist_in_6[441:429] = poly_twist_in_5_buffer[441:429] ^ poly_twist_in_5_buffer[454:442] ^ poly_twist_in_5_buffer[467:455];
assign poly_twist_in_6[428:416] = poly_twist_in_5_buffer[428:416];
assign poly_twist_in_6[415:403] = poly_twist_in_5_buffer[415:403];
assign poly_twist_in_6[402:390] = poly_twist_in_5_buffer[402:390] ^ poly_twist_in_5_buffer[415:403];
assign poly_twist_in_6[389:377] = poly_twist_in_5_buffer[389:377] ^ poly_twist_in_5_buffer[402:390] ^ poly_twist_in_5_buffer[415:403];
assign poly_twist_in_6[376:364] = poly_twist_in_5_buffer[376:364];
assign poly_twist_in_6[363:351] = poly_twist_in_5_buffer[363:351];
assign poly_twist_in_6[350:338] = poly_twist_in_5_buffer[350:338] ^ poly_twist_in_5_buffer[363:351];
assign poly_twist_in_6[337:325] = poly_twist_in_5_buffer[337:325] ^ poly_twist_in_5_buffer[350:338] ^ poly_twist_in_5_buffer[363:351];
assign poly_twist_in_6[324:312] = poly_twist_in_5_buffer[324:312];
assign poly_twist_in_6[311:299] = poly_twist_in_5_buffer[311:299];
assign poly_twist_in_6[298:286] = poly_twist_in_5_buffer[298:286] ^ poly_twist_in_5_buffer[311:299];
assign poly_twist_in_6[285:273] = poly_twist_in_5_buffer[285:273] ^ poly_twist_in_5_buffer[298:286] ^ poly_twist_in_5_buffer[311:299];
assign poly_twist_in_6[272:260] = poly_twist_in_5_buffer[272:260];
assign poly_twist_in_6[259:247] = poly_twist_in_5_buffer[259:247];
assign poly_twist_in_6[246:234] = poly_twist_in_5_buffer[246:234] ^ poly_twist_in_5_buffer[259:247];
assign poly_twist_in_6[233:221] = poly_twist_in_5_buffer[233:221] ^ poly_twist_in_5_buffer[246:234] ^ poly_twist_in_5_buffer[259:247];
assign poly_twist_in_6[220:208] = poly_twist_in_5_buffer[220:208];
assign poly_twist_in_6[207:195] = poly_twist_in_5_buffer[207:195];
assign poly_twist_in_6[194:182] = poly_twist_in_5_buffer[194:182] ^ poly_twist_in_5_buffer[207:195];
assign poly_twist_in_6[181:169] = poly_twist_in_5_buffer[181:169] ^ poly_twist_in_5_buffer[194:182] ^ poly_twist_in_5_buffer[207:195];
assign poly_twist_in_6[168:156] = poly_twist_in_5_buffer[168:156];
assign poly_twist_in_6[155:143] = poly_twist_in_5_buffer[155:143];
assign poly_twist_in_6[142:130] = poly_twist_in_5_buffer[142:130] ^ poly_twist_in_5_buffer[155:143];
assign poly_twist_in_6[129:117] = poly_twist_in_5_buffer[129:117] ^ poly_twist_in_5_buffer[142:130] ^ poly_twist_in_5_buffer[155:143];
assign poly_twist_in_6[116:104] = poly_twist_in_5_buffer[116:104];
assign poly_twist_in_6[103:91] = poly_twist_in_5_buffer[103:91];
assign poly_twist_in_6[90:78] = poly_twist_in_5_buffer[90:78] ^ poly_twist_in_5_buffer[103:91];
assign poly_twist_in_6[77:65] = poly_twist_in_5_buffer[77:65] ^ poly_twist_in_5_buffer[90:78] ^ poly_twist_in_5_buffer[103:91];
assign poly_twist_in_6[64:52] = poly_twist_in_5_buffer[64:52];
assign poly_twist_in_6[51:39] = poly_twist_in_5_buffer[51:39];
assign poly_twist_in_6[38:26] = poly_twist_in_5_buffer[38:26] ^ poly_twist_in_5_buffer[51:39];
assign poly_twist_in_6[25:13] = poly_twist_in_5_buffer[25:13] ^ poly_twist_in_5_buffer[38:26] ^ poly_twist_in_5_buffer[51:39];
assign poly_twist_in_6[12:0] = poly_twist_in_5_buffer[12:0];


always @(posedge clk)
  begin
    poly_twist_in_1_buffer <= poly_twist_in_1;
    poly_twist_in_2_buffer <= poly_twist_in_2;
    poly_twist_in_3_buffer <= poly_twist_in_3;
    poly_twist_in_4_buffer <= poly_twist_in_4;
    poly_twist_in_5_buffer <= poly_twist_in_5;
    poly_twist_in_6_buffer <= poly_twist_in_6;
  end

wire round_is_0_buffer;
wire round_is_1_buffer;
wire round_is_2_buffer;
wire round_is_3_buffer;
wire round_is_4_buffer;
wire round_is_5_buffer;

assign round_is_0_buffer = (round == 0);
assign round_is_1_buffer = (round == 1);
assign round_is_2_buffer = (round == 2);
assign round_is_3_buffer = (round == 3);
assign round_is_4_buffer = (round == 4);
assign round_is_5_buffer = (round == 5);

reg round_is_0 = 1'b0;
reg round_is_1 = 1'b0;
reg round_is_2 = 1'b0;
reg round_is_3 = 1'b0;
reg round_is_4 = 1'b0;
reg round_is_5 = 1'b0;

always @(posedge clk)
  begin
    round_is_0 <= round_is_0_buffer;
    round_is_1 <= round_is_1_buffer;
    round_is_2 <= round_is_2_buffer;
    round_is_3 <= round_is_3_buffer;
    round_is_4 <= round_is_4_buffer;
    round_is_5 <= round_is_5_buffer;
  end

assign poly_out = round_is_0 ? poly_twist_in_6_buffer :
                  round_is_1 ? poly_twist_in_5_buffer :
                  round_is_2 ? poly_twist_in_4_buffer :
                  round_is_3 ? poly_twist_in_3_buffer :
                  round_is_4 ? poly_twist_in_2_buffer :
                  poly_twist_in_1_buffer;

reg [4:0] counter = 5'b0;
always @(posedge clk)
  begin
    counter <= (start & (round < 5)) ? counter + 1 :
               done ? 5'b0 :
               (counter > 5'b0) ? counter + 1 :
               counter;
  end


reg done_buffer = 0;

always @(posedge clk)
  begin
    done_buffer <= (round == 5) ? start :
                  ((round == 4) | (round == 3)) ? (counter == 1) :
                  ((round == 2) | (round == 1)) ? (counter == 2) :
                  (round == 0) ? (counter == 3) :
                   1'b0;
  end

assign done = done_buffer;
endmodule
