/*
 * Copyright (C) 2017
 * Authors: Wen Wang <wen.wang.ww349@yale.edu>
 *          Ruben Niederhagen <ruben@polycephaly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
*/


// reduction part in additive FFT module

module reduction (
  input  wire         clk,
  input  wire         start,
  input  wire [1663:0] coeff_in,
  input  wire         rd_en,
  input  wire [6:0] rd_addr,
  output wire [415:0] mem_coeff_A_dout_0,
  output wire [415:0] mem_coeff_B_dout_0,
  output wire         done
);

//rearrange the coefficients based on reversal list

wire [1663:0] coeff_re;

assign coeff_re[12:0] = coeff_in[12:0];
assign coeff_re[25:13] = coeff_in[844:832];
assign coeff_re[38:26] = coeff_in[428:416];
assign coeff_re[51:39] = coeff_in[1260:1248];
assign coeff_re[64:52] = coeff_in[220:208];
assign coeff_re[77:65] = coeff_in[1052:1040];
assign coeff_re[90:78] = coeff_in[636:624];
assign coeff_re[103:91] = coeff_in[1468:1456];
assign coeff_re[116:104] = coeff_in[116:104];
assign coeff_re[129:117] = coeff_in[948:936];
assign coeff_re[142:130] = coeff_in[532:520];
assign coeff_re[155:143] = coeff_in[1364:1352];
assign coeff_re[168:156] = coeff_in[324:312];
assign coeff_re[181:169] = coeff_in[1156:1144];
assign coeff_re[194:182] = coeff_in[740:728];
assign coeff_re[207:195] = coeff_in[1572:1560];
assign coeff_re[220:208] = coeff_in[64:52];
assign coeff_re[233:221] = coeff_in[896:884];
assign coeff_re[246:234] = coeff_in[480:468];
assign coeff_re[259:247] = coeff_in[1312:1300];
assign coeff_re[272:260] = coeff_in[272:260];
assign coeff_re[285:273] = coeff_in[1104:1092];
assign coeff_re[298:286] = coeff_in[688:676];
assign coeff_re[311:299] = coeff_in[1520:1508];
assign coeff_re[324:312] = coeff_in[168:156];
assign coeff_re[337:325] = coeff_in[1000:988];
assign coeff_re[350:338] = coeff_in[584:572];
assign coeff_re[363:351] = coeff_in[1416:1404];
assign coeff_re[376:364] = coeff_in[376:364];
assign coeff_re[389:377] = coeff_in[1208:1196];
assign coeff_re[402:390] = coeff_in[792:780];
assign coeff_re[415:403] = coeff_in[1624:1612];
assign coeff_re[428:416] = coeff_in[38:26];
assign coeff_re[441:429] = coeff_in[870:858];
assign coeff_re[454:442] = coeff_in[454:442];
assign coeff_re[467:455] = coeff_in[1286:1274];
assign coeff_re[480:468] = coeff_in[246:234];
assign coeff_re[493:481] = coeff_in[1078:1066];
assign coeff_re[506:494] = coeff_in[662:650];
assign coeff_re[519:507] = coeff_in[1494:1482];
assign coeff_re[532:520] = coeff_in[142:130];
assign coeff_re[545:533] = coeff_in[974:962];
assign coeff_re[558:546] = coeff_in[558:546];
assign coeff_re[571:559] = coeff_in[1390:1378];
assign coeff_re[584:572] = coeff_in[350:338];
assign coeff_re[597:585] = coeff_in[1182:1170];
assign coeff_re[610:598] = coeff_in[766:754];
assign coeff_re[623:611] = coeff_in[1598:1586];
assign coeff_re[636:624] = coeff_in[90:78];
assign coeff_re[649:637] = coeff_in[922:910];
assign coeff_re[662:650] = coeff_in[506:494];
assign coeff_re[675:663] = coeff_in[1338:1326];
assign coeff_re[688:676] = coeff_in[298:286];
assign coeff_re[701:689] = coeff_in[1130:1118];
assign coeff_re[714:702] = coeff_in[714:702];
assign coeff_re[727:715] = coeff_in[1546:1534];
assign coeff_re[740:728] = coeff_in[194:182];
assign coeff_re[753:741] = coeff_in[1026:1014];
assign coeff_re[766:754] = coeff_in[610:598];
assign coeff_re[779:767] = coeff_in[1442:1430];
assign coeff_re[792:780] = coeff_in[402:390];
assign coeff_re[805:793] = coeff_in[1234:1222];
assign coeff_re[818:806] = coeff_in[818:806];
assign coeff_re[831:819] = coeff_in[1650:1638];
assign coeff_re[844:832] = coeff_in[25:13];
assign coeff_re[857:845] = coeff_in[857:845];
assign coeff_re[870:858] = coeff_in[441:429];
assign coeff_re[883:871] = coeff_in[1273:1261];
assign coeff_re[896:884] = coeff_in[233:221];
assign coeff_re[909:897] = coeff_in[1065:1053];
assign coeff_re[922:910] = coeff_in[649:637];
assign coeff_re[935:923] = coeff_in[1481:1469];
assign coeff_re[948:936] = coeff_in[129:117];
assign coeff_re[961:949] = coeff_in[961:949];
assign coeff_re[974:962] = coeff_in[545:533];
assign coeff_re[987:975] = coeff_in[1377:1365];
assign coeff_re[1000:988] = coeff_in[337:325];
assign coeff_re[1013:1001] = coeff_in[1169:1157];
assign coeff_re[1026:1014] = coeff_in[753:741];
assign coeff_re[1039:1027] = coeff_in[1585:1573];
assign coeff_re[1052:1040] = coeff_in[77:65];
assign coeff_re[1065:1053] = coeff_in[909:897];
assign coeff_re[1078:1066] = coeff_in[493:481];
assign coeff_re[1091:1079] = coeff_in[1325:1313];
assign coeff_re[1104:1092] = coeff_in[285:273];
assign coeff_re[1117:1105] = coeff_in[1117:1105];
assign coeff_re[1130:1118] = coeff_in[701:689];
assign coeff_re[1143:1131] = coeff_in[1533:1521];
assign coeff_re[1156:1144] = coeff_in[181:169];
assign coeff_re[1169:1157] = coeff_in[1013:1001];
assign coeff_re[1182:1170] = coeff_in[597:585];
assign coeff_re[1195:1183] = coeff_in[1429:1417];
assign coeff_re[1208:1196] = coeff_in[389:377];
assign coeff_re[1221:1209] = coeff_in[1221:1209];
assign coeff_re[1234:1222] = coeff_in[805:793];
assign coeff_re[1247:1235] = coeff_in[1637:1625];
assign coeff_re[1260:1248] = coeff_in[51:39];
assign coeff_re[1273:1261] = coeff_in[883:871];
assign coeff_re[1286:1274] = coeff_in[467:455];
assign coeff_re[1299:1287] = coeff_in[1299:1287];
assign coeff_re[1312:1300] = coeff_in[259:247];
assign coeff_re[1325:1313] = coeff_in[1091:1079];
assign coeff_re[1338:1326] = coeff_in[675:663];
assign coeff_re[1351:1339] = coeff_in[1507:1495];
assign coeff_re[1364:1352] = coeff_in[155:143];
assign coeff_re[1377:1365] = coeff_in[987:975];
assign coeff_re[1390:1378] = coeff_in[571:559];
assign coeff_re[1403:1391] = coeff_in[1403:1391];
assign coeff_re[1416:1404] = coeff_in[363:351];
assign coeff_re[1429:1417] = coeff_in[1195:1183];
assign coeff_re[1442:1430] = coeff_in[779:767];
assign coeff_re[1455:1443] = coeff_in[1611:1599];
assign coeff_re[1468:1456] = coeff_in[103:91];
assign coeff_re[1481:1469] = coeff_in[935:923];
assign coeff_re[1494:1482] = coeff_in[519:507];
assign coeff_re[1507:1495] = coeff_in[1351:1339];
assign coeff_re[1520:1508] = coeff_in[311:299];
assign coeff_re[1533:1521] = coeff_in[1143:1131];
assign coeff_re[1546:1534] = coeff_in[727:715];
assign coeff_re[1559:1547] = coeff_in[1559:1547];
assign coeff_re[1572:1560] = coeff_in[207:195];
assign coeff_re[1585:1573] = coeff_in[1039:1027];
assign coeff_re[1598:1586] = coeff_in[623:611];
assign coeff_re[1611:1599] = coeff_in[1455:1443];
assign coeff_re[1624:1612] = coeff_in[415:403];
assign coeff_re[1637:1625] = coeff_in[1247:1235];
assign coeff_re[1650:1638] = coeff_in[831:819];
assign coeff_re[1663:1651] = coeff_in[1663:1651];

// initialize mem_coeff_A and mem_coeff_B

reg [5:0] init_counter = 6'b0;
reg [5:0] init_counter_reg = 6'b0;

always @(posedge clk)
  begin
    init_counter <= start ? init_counter + 1 :
                    (init_counter > 0) ? init_counter + 1 :
                    init_counter;
    init_counter_reg <= init_counter;
  end
  
wire init_running;
reg init_running_reg = 1'b0;
reg init_running_reg_buffer = 1'b0;

assign init_running = start | (init_counter >= 1 && init_counter <= 63);

always @(posedge clk)
  begin
    init_running_reg <= init_running;
    init_running_reg_buffer <= init_running_reg;
  end

// generate coeff mem initialization contents
 
reg [1663:0] coeff_reg = 1664'b0;

reg [12:0] coeff_lowest = 13'b0;
reg [12:0] coeff_second_lowest = 13'b0;

always @(posedge clk)
  begin
    coeff_reg <= start ? coeff_re : (coeff_reg >> 26);
    
    coeff_lowest <= coeff_reg[12:0];
    coeff_second_lowest <= coeff_reg[25:13];
  end  

// evaluation counter for each round

reg [6:0] eva_counter_0 = 7'b0;

reg rearrange_done = 1'b0;
    
always @(posedge clk)
  begin
    eva_counter_0 <= rearrange_done ? eva_counter_0 + 1 :
                     (eva_counter_0 >= 1) ? eva_counter_0 + 1 :
                     eva_counter_0;
  end

wire round_0_running;
reg round_0_running_reg = 1'b0;
assign round_0_running = rearrange_done | (eva_counter_0 >= 1);
always @(posedge clk)
  begin
    round_0_running_reg <= round_0_running;
  end

reg [6:0] eva_counter_1 = 7'b0;

reg done_0 = 1'b0;
    
always @(posedge clk)
  begin
    eva_counter_1 <= done_0 ? eva_counter_1 + 1 :
                     (eva_counter_1 >= 1) ? eva_counter_1 + 1 :
                     eva_counter_1;
  end

wire round_1_running;
reg round_1_running_reg = 1'b0;
assign round_1_running = done_0 | (eva_counter_1 >= 1);
always @(posedge clk)
  begin
    round_1_running_reg <= round_1_running;
  end

reg [6:0] eva_counter_2 = 7'b0;

reg done_1 = 1'b0;
    
always @(posedge clk)
  begin
    eva_counter_2 <= done_1 ? eva_counter_2 + 1 :
                     (eva_counter_2 >= 1) ? eva_counter_2 + 1 :
                     eva_counter_2;
  end

wire round_2_running;
reg round_2_running_reg = 1'b0;
assign round_2_running = done_1 | (eva_counter_2 >= 1);
always @(posedge clk)
  begin
    round_2_running_reg <= round_2_running;
  end

reg [6:0] eva_counter_3 = 7'b0;

reg done_2 = 1'b0;
    
always @(posedge clk)
  begin
    eva_counter_3 <= done_2 ? eva_counter_3 + 1 :
                     (eva_counter_3 >= 1) ? eva_counter_3 + 1 :
                     eva_counter_3;
  end

wire round_3_running;
reg round_3_running_reg = 1'b0;
assign round_3_running = done_2 | (eva_counter_3 >= 1);
always @(posedge clk)
  begin
    round_3_running_reg <= round_3_running;
  end

reg [6:0] eva_counter_4 = 7'b0;

reg done_3 = 1'b0;
    
always @(posedge clk)
  begin
    eva_counter_4 <= done_3 ? eva_counter_4 + 1 :
                     (eva_counter_4 >= 1) ? eva_counter_4 + 1 :
                     eva_counter_4;
  end

wire round_4_running;
reg round_4_running_reg = 1'b0;
assign round_4_running = done_3 | (eva_counter_4 >= 1);
always @(posedge clk)
  begin
    round_4_running_reg <= round_4_running;
  end

reg [6:0] eva_counter_5 = 7'b0;

reg done_4 = 1'b0;
    
always @(posedge clk)
  begin
    eva_counter_5 <= done_4 ? eva_counter_5 + 1 :
                     (eva_counter_5 >= 1) ? eva_counter_5 + 1 :
                     eva_counter_5;
  end

wire round_5_running;
reg round_5_running_reg = 1'b0;
assign round_5_running = done_4 | (eva_counter_5 >= 1);
always @(posedge clk)
  begin
    round_5_running_reg <= round_5_running;
  end

reg [6:0] eva_counter_6 = 7'b0;

reg done_5 = 1'b0;
    
always @(posedge clk)
  begin
    eva_counter_6 <= done_5 ? eva_counter_6 + 1 :
                     (eva_counter_6 >= 1) ? eva_counter_6 + 1 :
                     eva_counter_6;
  end

wire round_6_running;
reg round_6_running_reg = 1'b0;
assign round_6_running = done_5 | (eva_counter_6 >= 1);
always @(posedge clk)
  begin
    round_6_running_reg <= round_6_running;
  end



always @(posedge clk)
  begin
    rearrange_done <= (init_counter == 63) ? 1'b1 : 1'b0;
  end


// interface for mem_coeff

reg [6:0] mem_coeff_rd_address_0_round_0 = 7'b0;
reg [6:0] mem_coeff_rd_address_0_round_1 = 7'b0;
reg [6:0] mem_coeff_rd_address_0_round_2 = 7'b0;
reg [6:0] mem_coeff_rd_address_0_round_3 = 7'b0;
reg [6:0] mem_coeff_rd_address_0_round_4 = 7'b0;
reg [6:0] mem_coeff_rd_address_0_round_5 = 7'b0;
reg [6:0] mem_coeff_rd_address_0_round_6 = 7'b0;

always @(posedge clk)
  begin
    mem_coeff_rd_address_0_round_0 <= eva_counter_0;
    mem_coeff_rd_address_0_round_1 <= ((eva_counter_1 >> 2) << 2) + ((eva_counter_1 >> 1) % 2);
    mem_coeff_rd_address_0_round_2 <= ((eva_counter_2 >> 3) << 3) + ((eva_counter_2 >> 1) % 4);
    mem_coeff_rd_address_0_round_3 <= ((eva_counter_3 >> 4) << 4) + ((eva_counter_3 >> 1) % 8);
    mem_coeff_rd_address_0_round_4 <= ((eva_counter_4 >> 5) << 5) + ((eva_counter_4 >> 1) % 16);
    mem_coeff_rd_address_0_round_5 <= ((eva_counter_5 >> 6) << 6) + ((eva_counter_5 >> 1) % 32);
    mem_coeff_rd_address_0_round_6 <= ((eva_counter_6 >> 1) % 64);
  end

reg [6:0] mem_coeff_rd_address_1_round_0 = 7'b0;
reg [6:0] mem_coeff_rd_address_1_round_1 = 7'b0;
reg [6:0] mem_coeff_rd_address_1_round_2 = 7'b0;
reg [6:0] mem_coeff_rd_address_1_round_3 = 7'b0;
reg [6:0] mem_coeff_rd_address_1_round_4 = 7'b0;
reg [6:0] mem_coeff_rd_address_1_round_5 = 7'b0;
reg [6:0] mem_coeff_rd_address_1_round_6 = 7'b0;

always @(posedge clk)
  begin
    mem_coeff_rd_address_1_round_0 <= eva_counter_0 + 1;
    mem_coeff_rd_address_1_round_1 <= ((eva_counter_1 >> 2) << 2) + ((eva_counter_1 >> 1) % 2) + 2;
    mem_coeff_rd_address_1_round_2 <= ((eva_counter_2 >> 3) << 3) + ((eva_counter_2 >> 1) % 4) + 4;
    mem_coeff_rd_address_1_round_3 <= ((eva_counter_3 >> 4) << 4) + ((eva_counter_3 >> 1) % 8) + 8;
    mem_coeff_rd_address_1_round_4 <= ((eva_counter_4 >> 5) << 5) + ((eva_counter_4 >> 1) % 16) + 16;
    mem_coeff_rd_address_1_round_5 <= ((eva_counter_5 >> 6) << 6) + ((eva_counter_5 >> 1) % 32) + 32;
    mem_coeff_rd_address_1_round_6 <= ((eva_counter_6 >> 1) % 64) + 64;
end

reg [6:0] mem_coeff_rd_address_0 = 7'b0;
reg [6:0] mem_coeff_rd_address_1 = 7'b0;

reg [6:0] mem_coeff_wr_address_0 = 7'b0;
reg [6:0] mem_coeff_wr_address_1 = 7'b0;


always @(posedge clk)
  begin
    mem_coeff_rd_address_0 <= round_0_running_reg ? mem_coeff_rd_address_0_round_0 :
                              round_1_running_reg ? mem_coeff_rd_address_0_round_1 :
                              round_2_running_reg ? mem_coeff_rd_address_0_round_2 :
                              round_3_running_reg ? mem_coeff_rd_address_0_round_3 :
                              round_4_running_reg ? mem_coeff_rd_address_0_round_4 :
                              round_5_running_reg ? mem_coeff_rd_address_0_round_5 :
                              round_6_running_reg ? mem_coeff_rd_address_0_round_6 :
                              7'b0;

    mem_coeff_rd_address_1 <= round_0_running_reg ? mem_coeff_rd_address_1_round_0 :
                              round_1_running_reg ? mem_coeff_rd_address_1_round_1 :
                              round_2_running_reg ? mem_coeff_rd_address_1_round_2 :
                              round_3_running_reg ? mem_coeff_rd_address_1_round_3 :
                              round_4_running_reg ? mem_coeff_rd_address_1_round_4 :
                              round_5_running_reg ? mem_coeff_rd_address_1_round_5 :
                              round_6_running_reg ? mem_coeff_rd_address_1_round_6 :
                              7'b0;
  end
reg [6:0] mem_coeff_rd_address_0_buffer_1 = 7'b0;
reg [6:0] mem_coeff_rd_address_0_buffer_2 = 7'b0;
reg [6:0] mem_coeff_rd_address_0_buffer_3 = 7'b0;
reg [6:0] mem_coeff_rd_address_0_buffer_4 = 7'b0;

reg [6:0] mem_coeff_rd_address_1_buffer_1 = 7'b0;
reg [6:0] mem_coeff_rd_address_1_buffer_2 = 7'b0;
reg [6:0] mem_coeff_rd_address_1_buffer_3 = 7'b0;
reg [6:0] mem_coeff_rd_address_1_buffer_4 = 7'b0;

always @(posedge clk)
  begin
    mem_coeff_wr_address_0 <= init_running_reg ? (init_counter_reg << 1) : mem_coeff_rd_address_0_buffer_4; 
    mem_coeff_wr_address_1 <= init_running_reg ? ((init_counter_reg << 1) + 1) : mem_coeff_rd_address_1_buffer_4; 
  end


reg [6:0] mem_coeff_address_0 = 7'b0;
reg [6:0] mem_coeff_address_1 = 7'b0;

always @(posedge clk)
  begin
    mem_coeff_rd_address_0_buffer_1 <= mem_coeff_rd_address_0;
    mem_coeff_rd_address_0_buffer_2 <= mem_coeff_rd_address_0_buffer_1;
    mem_coeff_rd_address_0_buffer_3 <= mem_coeff_rd_address_0_buffer_2;
    mem_coeff_rd_address_0_buffer_4 <= mem_coeff_rd_address_0_buffer_3;

    mem_coeff_rd_address_1_buffer_1 <= mem_coeff_rd_address_1;
    mem_coeff_rd_address_1_buffer_2 <= mem_coeff_rd_address_1_buffer_1;
    mem_coeff_rd_address_1_buffer_3 <= mem_coeff_rd_address_1_buffer_2;
    mem_coeff_rd_address_1_buffer_4 <= mem_coeff_rd_address_1_buffer_3;
  end

reg odd_signal_buffer_1 = 1'b0;
reg odd_signal_buffer_2 = 1'b0;
reg odd_signal_buffer_3 = 1'b0;
reg odd_signal_buffer_4 = 1'b0;
reg odd_signal_buffer_5 = 1'b0;


always @(posedge clk)
  begin
    mem_coeff_address_0 <= (init_running_reg_buffer | odd_signal_buffer_5) ? mem_coeff_wr_address_0 : mem_coeff_rd_address_0;
    mem_coeff_address_1 <= (init_running_reg_buffer | odd_signal_buffer_5) ? mem_coeff_wr_address_1 : mem_coeff_rd_address_1;
  end

wire [6:0] mem_coeff_A_address_0;
wire [6:0] mem_coeff_A_address_1;

reg [6:0] mem_coeff_B_address_0_buffer = 7'b0;
reg [6:0] mem_coeff_B_address_1_buffer = 7'b0;

wire [6:0] mem_coeff_B_address_0;
wire [6:0] mem_coeff_B_address_1;

always @(posedge clk)
  begin
    mem_coeff_B_address_0_buffer <= mem_coeff_address_0;
    mem_coeff_B_address_1_buffer <= mem_coeff_address_1;
  end

assign mem_coeff_A_address_0 = mem_coeff_address_0;
assign mem_coeff_A_address_1 = mem_coeff_address_1;

assign mem_coeff_B_address_0 = mem_coeff_B_address_0_buffer;
assign mem_coeff_B_address_1 = mem_coeff_B_address_1_buffer;

wire [415:0] mem_coeff_A_dout_1;

wire [415:0] mem_coeff_B_dout_1;

reg [415:0] mem_coeff_A_dout_0_reg = 416'b0;
reg [415:0] mem_coeff_A_dout_0_reg_buffer = 416'b0;

reg [415:0] mem_coeff_A_dout_1_reg = 416'b0;
reg [415:0] mem_coeff_A_dout_1_reg_buffer_1 = 416'b0;
reg [415:0] mem_coeff_A_dout_1_reg_buffer_2 = 416'b0;

reg [415:0] mem_coeff_B_dout_0_reg = 416'b0;
reg [415:0] mem_coeff_B_dout_0_reg_buffer = 416'b0;

reg [415:0] mem_coeff_B_dout_1_reg = 416'b0;
reg [415:0] mem_coeff_B_dout_1_reg_buffer_1 = 416'b0;
reg [415:0] mem_coeff_B_dout_1_reg_buffer_2 = 416'b0;

always @(posedge clk)
  begin
    mem_coeff_A_dout_0_reg <= mem_coeff_A_dout_0;
    mem_coeff_A_dout_0_reg_buffer <= mem_coeff_A_dout_0_reg;

    mem_coeff_A_dout_1_reg <= mem_coeff_A_dout_1;
    mem_coeff_A_dout_1_reg_buffer_1 <= mem_coeff_A_dout_1_reg;
    mem_coeff_A_dout_1_reg_buffer_2 <= mem_coeff_A_dout_1_reg_buffer_1;

    mem_coeff_B_dout_0_reg <= mem_coeff_B_dout_0;
    mem_coeff_B_dout_0_reg_buffer <= mem_coeff_B_dout_0_reg;

    mem_coeff_B_dout_1_reg <= mem_coeff_B_dout_1;
    mem_coeff_B_dout_1_reg_buffer_1 <= mem_coeff_B_dout_1_reg;
    mem_coeff_B_dout_1_reg_buffer_2 <= mem_coeff_B_dout_1_reg_buffer_1;	 
  end

reg mem_coeff_wr_en = 1'b0;

reg odd_signal = 1'b0;

always @(posedge clk)
  begin
    odd_signal <= (eva_counter_0 % 2 == 1) | (eva_counter_1 % 2 == 1) | (eva_counter_2 % 2 == 1) | (eva_counter_3 % 2 == 1) | (eva_counter_4 % 2 == 1) | (eva_counter_5 % 2 == 1) | (eva_counter_6 % 2 == 1);

    odd_signal_buffer_1 <= odd_signal;
    odd_signal_buffer_2 <= odd_signal_buffer_1;
    odd_signal_buffer_3 <= odd_signal_buffer_2;
    odd_signal_buffer_4 <= odd_signal_buffer_3;
    odd_signal_buffer_5 <= odd_signal_buffer_4;
    
    mem_coeff_wr_en <= init_running_reg_buffer | odd_signal_buffer_5;
  end

wire mem_coeff_A_wr_en;
reg mem_coeff_B_wr_en_buffer = 1'b0;
wire mem_coeff_B_wr_en;

assign mem_coeff_A_wr_en = mem_coeff_wr_en;

always @(posedge clk)
  begin
    mem_coeff_B_wr_en_buffer <= mem_coeff_wr_en;
  end

assign mem_coeff_B_wr_en = mem_coeff_B_wr_en_buffer;

wire [415:0] vec_c;
reg [415:0] vec_c_reg = 416'b0;


reg [415:0] mem_coeff_A_din_0 = 416'b0;
reg [415:0] mem_coeff_A_din_1 = 416'b0;

wire [415:0] mem_coeff_A_din_0_buffer;
wire [415:0] mem_coeff_A_din_1_buffer;

reg [415:0] mem_coeff_A_din_0_buffer_reg = 416'b0;

assign mem_coeff_A_din_0_buffer = vec_c_reg ^ mem_coeff_A_dout_0_reg_buffer;
assign mem_coeff_A_din_1_buffer = mem_coeff_A_din_0_buffer_reg ^ mem_coeff_A_dout_1_reg_buffer_2;


always @(posedge clk)
  begin
    if(init_running_reg_buffer)
      begin
         
        mem_coeff_A_din_0[12:0] <= coeff_lowest;
        mem_coeff_A_din_1[12:0] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[25:13] <= coeff_lowest;
        mem_coeff_A_din_1[25:13] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[38:26] <= coeff_lowest;
        mem_coeff_A_din_1[38:26] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[51:39] <= coeff_lowest;
        mem_coeff_A_din_1[51:39] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[64:52] <= coeff_lowest;
        mem_coeff_A_din_1[64:52] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[77:65] <= coeff_lowest;
        mem_coeff_A_din_1[77:65] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[90:78] <= coeff_lowest;
        mem_coeff_A_din_1[90:78] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[103:91] <= coeff_lowest;
        mem_coeff_A_din_1[103:91] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[116:104] <= coeff_lowest;
        mem_coeff_A_din_1[116:104] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[129:117] <= coeff_lowest;
        mem_coeff_A_din_1[129:117] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[142:130] <= coeff_lowest;
        mem_coeff_A_din_1[142:130] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[155:143] <= coeff_lowest;
        mem_coeff_A_din_1[155:143] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[168:156] <= coeff_lowest;
        mem_coeff_A_din_1[168:156] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[181:169] <= coeff_lowest;
        mem_coeff_A_din_1[181:169] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[194:182] <= coeff_lowest;
        mem_coeff_A_din_1[194:182] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[207:195] <= coeff_lowest;
        mem_coeff_A_din_1[207:195] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[220:208] <= coeff_lowest;
        mem_coeff_A_din_1[220:208] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[233:221] <= coeff_lowest;
        mem_coeff_A_din_1[233:221] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[246:234] <= coeff_lowest;
        mem_coeff_A_din_1[246:234] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[259:247] <= coeff_lowest;
        mem_coeff_A_din_1[259:247] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[272:260] <= coeff_lowest;
        mem_coeff_A_din_1[272:260] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[285:273] <= coeff_lowest;
        mem_coeff_A_din_1[285:273] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[298:286] <= coeff_lowest;
        mem_coeff_A_din_1[298:286] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[311:299] <= coeff_lowest;
        mem_coeff_A_din_1[311:299] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[324:312] <= coeff_lowest;
        mem_coeff_A_din_1[324:312] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[337:325] <= coeff_lowest;
        mem_coeff_A_din_1[337:325] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[350:338] <= coeff_lowest;
        mem_coeff_A_din_1[350:338] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[363:351] <= coeff_lowest;
        mem_coeff_A_din_1[363:351] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[376:364] <= coeff_lowest;
        mem_coeff_A_din_1[376:364] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[389:377] <= coeff_lowest;
        mem_coeff_A_din_1[389:377] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[402:390] <= coeff_lowest;
        mem_coeff_A_din_1[402:390] <= coeff_second_lowest;
         
         
        mem_coeff_A_din_0[415:403] <= coeff_lowest;
        mem_coeff_A_din_1[415:403] <= coeff_second_lowest;
         
      end
    else
      begin
        mem_coeff_A_din_0_buffer_reg <= mem_coeff_A_din_0_buffer;
        mem_coeff_A_din_0 <= mem_coeff_A_din_0_buffer_reg;
        mem_coeff_A_din_1 <= mem_coeff_A_din_1_buffer;
      end
  end

reg [415:0] mem_coeff_B_din_0 = 416'b0;
reg [415:0] mem_coeff_B_din_1 = 416'b0;

wire [415:0] mem_coeff_B_din_0_buffer;
wire [415:0] mem_coeff_B_din_1_buffer;

reg [415:0] mem_coeff_B_din_0_buffer_reg = 416'b0;

assign mem_coeff_B_din_0_buffer = vec_c_reg ^ mem_coeff_B_dout_0_reg_buffer;
assign mem_coeff_B_din_1_buffer = mem_coeff_B_din_0_buffer_reg ^ mem_coeff_B_dout_1_reg_buffer_2;


reg [415:0] mem_coeff_B_init_din_0_buffer = 416'b0;
reg [415:0] mem_coeff_B_init_din_1_buffer = 416'b0;
reg init_running_reg_buffer_B = 1'b0;

always @(posedge clk)
  begin
    init_running_reg_buffer_B <= init_running_reg_buffer;
    
    mem_coeff_B_init_din_0_buffer <= coeff_lowest;
    mem_coeff_B_init_din_1_buffer <= coeff_second_lowest;
    if(init_running_reg_buffer_B)
      begin
        mem_coeff_B_din_0[12:0] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[12:0] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[25:13] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[25:13] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[38:26] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[38:26] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[51:39] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[51:39] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[64:52] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[64:52] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[77:65] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[77:65] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[90:78] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[90:78] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[103:91] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[103:91] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[116:104] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[116:104] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[129:117] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[129:117] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[142:130] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[142:130] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[155:143] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[155:143] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[168:156] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[168:156] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[181:169] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[181:169] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[194:182] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[194:182] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[207:195] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[207:195] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[220:208] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[220:208] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[233:221] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[233:221] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[246:234] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[246:234] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[259:247] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[259:247] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[272:260] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[272:260] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[285:273] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[285:273] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[298:286] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[298:286] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[311:299] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[311:299] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[324:312] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[324:312] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[337:325] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[337:325] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[350:338] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[350:338] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[363:351] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[363:351] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[376:364] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[376:364] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[389:377] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[389:377] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[402:390] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[402:390] <= mem_coeff_B_init_din_1_buffer;
        mem_coeff_B_din_0[415:403] <= mem_coeff_B_init_din_0_buffer;
        mem_coeff_B_din_1[415:403] <= mem_coeff_B_init_din_1_buffer;
      end
    else
      begin
        mem_coeff_B_din_0_buffer_reg <= mem_coeff_B_din_0_buffer;
        mem_coeff_B_din_0 <= mem_coeff_B_din_0_buffer_reg;
        mem_coeff_B_din_1 <= mem_coeff_B_din_1_buffer;
      end
  end

wire [831:0] mem_consts_dout;

// interface for vec_mul module
// time mux the vector multiplier

reg [415:0] vec_a_buffer = 416'b0;
wire [415:0] vec_a;
always @(posedge clk)
  begin
    vec_a_buffer <= odd_signal_buffer_2 ? mem_coeff_A_dout_1 : mem_coeff_B_dout_1;
  end
assign vec_a = vec_a_buffer;

reg [415:0] vec_b_buffer = 416'b0;
wire [415:0] vec_b;
reg [415:0] mem_consts_B_reg = 416'b0;

always @(posedge clk)
  begin
    mem_consts_B_reg <= mem_consts_dout[831:416];  
    vec_b_buffer <= odd_signal_buffer_2 ? mem_consts_dout[415:0] : mem_consts_B_reg;
  end
assign vec_b = vec_b_buffer;

always @(posedge clk)
  begin
    vec_c_reg <= vec_c;
  end

// interface for mem_consts

wire mem_consts_rd_en;
assign mem_consts_rd_en = 1'b1;

reg [6:0] mem_consts_rdaddress_buffer = 7'b0;
reg [6:0] mem_consts_rdaddress = 7'b0;

always @(posedge clk)
  begin
    mem_consts_rdaddress_buffer <= round_0_running_reg ? 7'b0 :
                                   round_1_running_reg ? 1 + ((eva_counter_1 % 4) >> 1) :
                                   round_2_running_reg ? 3 + ((eva_counter_2 % 8) >> 1) :
                                   round_3_running_reg ? 7 + ((eva_counter_3 % 16) >> 1) :
                                   round_4_running_reg ? 15 + ((eva_counter_4 % 32) >> 1) :
                                   round_5_running_reg ? 31 + ((eva_counter_5 % 64) >> 1) :
                                   round_6_running_reg ? 63 + (eva_counter_6 >> 1) :
                                   7'b0;
    mem_consts_rdaddress <= mem_consts_rdaddress_buffer;
  end


// done signals for each evaluation round
reg done_6 = 1'b0;

always @(posedge clk)
  begin
    done_0 <= start ? 1'b0 :
                (eva_counter_0 == 127) ? 1'b1 :
                1'b0;
  end

always @(posedge clk)
  begin
    done_1 <= start ? 1'b0 :
                (eva_counter_1 == 127) ? 1'b1 :
                1'b0;
  end

always @(posedge clk)
  begin
    done_2 <= start ? 1'b0 :
                (eva_counter_2 == 127) ? 1'b1 :
                1'b0;
  end

always @(posedge clk)
  begin
    done_3 <= start ? 1'b0 :
                (eva_counter_3 == 127) ? 1'b1 :
                1'b0;
  end

always @(posedge clk)
  begin
    done_4 <= start ? 1'b0 :
                (eva_counter_4 == 127) ? 1'b1 :
                1'b0;
  end

always @(posedge clk)
  begin
    done_5 <= start ? 1'b0 :
                (eva_counter_5 == 127) ? 1'b1 :
                1'b0;
  end

always @(posedge clk)
  begin
    done_6 <= start ? 1'b0 :
                (eva_counter_6 == 127) ? 1'b1 :
                1'b0;
  end

reg done_reg_0 = 1'b0;
reg done_reg_1 = 1'b0;
reg done_reg_2 = 1'b0;
reg done_reg_3 = 1'b0;
reg done_reg_4 = 1'b0;
reg done_reg_5 = 1'b0;
reg done_reg_6 = 1'b0;
reg done_buffer = 1'b0;

always @(posedge clk)
  begin
    done_reg_0 <= done_6;
    done_reg_1 <= done_reg_0;
    done_reg_2 <= done_reg_1;
    done_reg_3 <= done_reg_2;
    done_reg_4 <= done_reg_3;
    done_reg_5 <= done_reg_4;
    done_reg_6 <= done_reg_5;
    done_buffer <= done_reg_6;
  end

assign done = done_buffer;

// instances declaration

// vec_mul module

vec_mul_reduction vec_mul_reduction_inst (
  .clk(clk),
  .vec_a(vec_a),
  .vec_b(vec_b),
  .vec_c(vec_c)
);

// two dual-port mem modules for storing evaluation data

mem_dual #(.WIDTH(416), .DEPTH(128)) mem_coeff_A (
  .clock (clk),
  .data_0 (mem_coeff_A_din_0),
  .data_1 (mem_coeff_A_din_1),
  .address_0 (rd_en ? rd_addr : mem_coeff_A_address_0),
  .address_1 (mem_coeff_A_address_1),
  .wren_0 (mem_coeff_A_wr_en),
  .wren_1 (mem_coeff_A_wr_en),
  .q_0 (mem_coeff_A_dout_0),
  .q_1 (mem_coeff_A_dout_1)
);
 
mem_dual #(.WIDTH(416), .DEPTH(128)) mem_coeff_B (
  .clock (clk),
  .data_0 (mem_coeff_B_din_0),
  .data_1 (mem_coeff_B_din_1),
  .address_0 (rd_en ? rd_addr : mem_coeff_B_address_0),
  .address_1 (mem_coeff_B_address_1),
  .wren_0 (mem_coeff_B_wr_en),
  .wren_1 (mem_coeff_B_wr_en),
  .q_0 (mem_coeff_B_dout_0),
  .q_1 (mem_coeff_B_dout_1)
);
 
// one single-port mem module for string constants data

mem #(.WIDTH(832), .DEPTH(128)) mem_consts (
  .clock (clk),
  .data (0),
  .rdaddress (mem_consts_rdaddress),
  .rden (mem_consts_rd_en),
  .wraddress (0),
  .wren (0),
  .q (mem_consts_dout)
);
 

// Use below defparam for PCIe compilation on DE5 board
//defparam mem_consts.FILE = "../../../pqc_enc_dec/src/keygen/key_gen/Quartus/mem_consts";

// Use below defparam for simulation
defparam mem_consts.FILE = "mem_consts";

endmodule


