/*
 * Copyright (C) 2017
 * Authors: Wen Wang <wen.wang.ww349@yale.edu>
 *          Ruben Niederhagen <ruben@polycephaly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
*/

module vec_mul_multi_H_gen (
 
  input  wire clk,
  input  wire start,
  input  wire [675:0] vec_a,
  input  wire [675:0] vec_b,
  output wire [675:0] vec_c,
  output wire done
   
);


reg [3:0] counter = 0;
reg running = 0;
reg running_sh = 0;

always @(posedge clk)
  begin
    running <= start ? 1'b1 : 
           (counter >= 12) ? 1'b0 : running;
    running_sh <= start ? 1'b1 : 
           (counter >= 11) ? 1'b0 : running;
  end

always @(posedge clk)
  begin
    counter <= start ? 0 :
               (running && (counter <= 12)) ? counter + 1 :
               counter;
  end

/*
reg done_buffer = 0;

always @(posedge clk)
  begin
    done_buffer <= (counter == 12) ? 1 : 0;
  end

assign done = done_buffer;
*/

assign done = (counter == 12);


reg [675:0] vec_a_reg = 0;
reg [675:0] vec_b_reg = 0;

always @(posedge clk)
  begin
    vec_a_reg <= start ? vec_a : (vec_a_reg >> 52);
    vec_b_reg <= start ? vec_b : (vec_b_reg >> 52);
  end


wire [51:0] A;
wire [51:0] B;
wire [51:0] C;
 
assign A = start ? vec_a[51:0] : (vec_a_reg >> 52);
assign B = start ? vec_b[51:0] : (vec_b_reg >> 52);


reg [675:0] vec_c_buffer = 0;

always @(posedge clk)
  begin
    vec_c_buffer <= start      ? {C, 624'b0} :
                    running_sh ? (vec_c_buffer >> 52) ^ {C, 624'b0} :
                    vec_c_buffer;
  end
 
assign vec_c = vec_c_buffer;


vec_mul_H_gen vec_mul_H_gen_inst (
  .clk(clk),
  .vec_a(A),
  .vec_b(B),
  .vec_c(C)
);

endmodule

