/*
 * Copyright (C) 2017
 * Authors: Wen Wang <wen.wang.ww349@yale.edu>
 *          Ruben Niederhagen <ruben@polycephaly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
*/

module vec_mul_multi_twist (
 
  input  wire clk,
  input  wire start,
  input  wire [1663:0] vec_a,
  input  wire [1663:0] vec_b,
  output wire [1663:0] vec_c,
  output wire done
   
);


reg [6:0] counter = 0;
reg running = 0;
reg running_sh = 0;

always @(posedge clk)
  begin
    running <= start ? 1'b1 : 
           (counter >= 63) ? 1'b0 : running;
    running_sh <= start ? 1'b1 : 
           (counter >= 62) ? 1'b0 : running;
  end

always @(posedge clk)
  begin
    counter <= start ? 0 :
               (running && (counter <= 63)) ? counter + 1 :
               counter;
  end

/*
reg done_buffer = 0;

always @(posedge clk)
  begin
    done_buffer <= (counter == 63) ? 1 : 0;
  end

assign done = done_buffer;
*/

assign done = (counter == 63);


reg [1663:0] vec_a_reg = 0;
reg [1663:0] vec_b_reg = 0;

always @(posedge clk)
  begin
    vec_a_reg <= start ? vec_a : (vec_a_reg >> 26);
    vec_b_reg <= start ? vec_b : (vec_b_reg >> 26);
  end


wire [25:0] A;
wire [25:0] B;
wire [25:0] C;
 
assign A = start ? vec_a[25:0] : (vec_a_reg >> 26);
assign B = start ? vec_b[25:0] : (vec_b_reg >> 26);


reg [1663:0] vec_c_buffer = 0;

always @(posedge clk)
  begin
    vec_c_buffer <= start      ? {C, 1638'b0} :
                    running_sh ? (vec_c_buffer >> 26) ^ {C, 1638'b0} :
                    vec_c_buffer;
  end
 
assign vec_c = vec_c_buffer;


vec_mul_twist vec_mul_twist_inst (
  .clk(clk),
  .vec_a(A),
  .vec_b(B),
  .vec_c(C)
);

endmodule

